/** TDSP *******************************************************************
                          TDSPAwgn.h  -  description
                             -------------------
    begin                : Mon Nov 19 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPAWGN_H
#define TDSPAWGN_H

#include <TRandom.h>
#include <TDSPOutput1.h>

/**Additive white gaussian noise
  *@author Peter Jung
  */

class TDSPAwgn : public TDSPOutput1  {

 private:
  
  Double_t fTime;         // number entries in the cumulated Input Signal Energy (depents on the sampling rate !)
  Double_t fInEnergy;     // cumulated Input Signal Energy
  Double_t fEnergy;       // cumulated Output Signal Energy
  Double_t fNoiseLevel;   // sigma of the random gaussian noise

 public: 
  TDSPAwgn(char *s=NULL);
  ~TDSPAwgn();

  void      SetNoiseLevel(Double_t n) { fNoiseLevel = n;};  // Set the Noise Level 
  void      SetNoisePower(Double_t n) { SetNoiseLevel(TMath::Sqrt(n));};  // Set the Noise Power
  Double_t  GetNoiseLevel() { return fNoiseLevel;};         // Get the Noise Level
  Double_t  GetNoisePower() { return fNoiseLevel*fNoiseLevel;};  // Get the Noise Power
  
  Double_t  GetInputSignalEnergy() { return fInEnergy;};  ///< E = x^*x * epsilon
  Double_t  GetTime() const { return fTime;}; ///< T = N * epsilon
  Double_t  GetInputSignalPower() { return fInEnergy/fTime;}; ///< P = (x^*x * epsilon)/(N * epsilon)=x^*x/N
  Double_t  GetOutputSignalEnergy() { return fEnergy; };
  Double_t  GetOutputSignalPower() { return fEnergy/fTime;};
  Double_t  GetSNR() { return GetInputSignalPower()/GetNoisePower();}; ///< snr = (x^*x * epsilon)/(N * epsilon)/sigma^2_N = x^*x/N/sigma^2_N
  

  // Slots
  TDSPSignal* Update(TDSPSignal*); // Update-Slot
  void        Reset();             // Reset-Slot

  ClassDef(TDSPAwgn,1)

};

#endif
