/** TDSP *******************************************************************
                          TDSPBlockFifo.h  -  description
                             -------------------
    begin                : Thu Jan 10 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPBLOCKFIFO_H
#define TDSPBLOCKFIFO_H

#include <TDSPFifo.h>
#include <TDSPOutput1.h>

/**2-Dim FIFO implementation
  *@author Peter Jung
  *
  * The FIFO of order M is designed in that way that you have not only
  * M past values avaiable but the current too !!
  *
  */

class TDSPBlockFifo : public TDSPOutput1, public TDSPFifo  {
  
  friend class TDSPFifoFilter;
  
 protected:

  Int_t     fWidth; //

 public: 
  
  TDSPBlockFifo(char *name = NULL);
  ~TDSPBlockFifo();

  TComplex *GetBlock(Int_t i=0) { return &fBlock[i*fWidth];};

  void  SetDepth(Int_t depth);
  void  SetSize(Int_t width, Int_t depth);
  Int_t GetWidth() { return fWidth;};
      
  TDSPSignal         *Shift(TDSPSignal *input = NULL, TDSPSignal *output = NULL);

  // Slots

  TDSPSignal *Update(TDSPSignal *input) {
    TDSPSignal *output=Shift(input,GetSignal());
    Emit("Update(TDSPSignal*)",output);
    return output;
  }

  ClassDef(TDSPBlockFifo,1)

};

inline TDSPSignal *TDSPBlockFifo::Shift(TDSPSignal *input, TDSPSignal *output) {

  register  Int_t i;

  Int_t     iLen = 0;
  TComplex *ivec = NULL;

  if (input) {
    iLen = input->GetLen();
    ivec = input->GetVec();

    // Use only fWidth values of the input Signal
    //
    if (iLen>fWidth) iLen = fWidth;
    
  }

  /////////////////////////////////////////
  //
  // Copy
  //
  TComplex *ovec = GetBlock(fIndex);
  

  for(i=0;   i<iLen;  i++) ovec[i] = ivec[i];
  for(   ;i<fWidth;i++)    ovec[i] = 0;

  // Increase the Index cyclic
  //
  Next();
  
  // Update the Fill Status
  //
  Fill();
  
  // Checking Fill Status of the Fifo
  //
  if (WasFull()) {
    
    // Set the Output Signal (if given)
    //
    if (output) output->SetVec(GetBlock(fIndex),fWidth);
    
  } else
    output = NULL;


  return output;

}


#endif
