/** TDSP *******************************************************************
                          TDSPBlockSolve.h  -  description
                             -------------------
    begin                : Mon Mar 18 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPBLOCKSOLVE_H
#define TDSPBLOCKSOLVE_H

#include <TDSPBlockOperation.h>
#include <TDSPOutput1.h>
#include <TDSPSolver.h>

/**applies a solver blockwise onto the
   data stream
   *@author Peter Jung
   */

class TDSPBlockSolve : public TDSPOutput1, public TDSPBlockOperation  {
  TDSPSolver* fSolver;
 public: 
  TDSPBlockSolve(char *name = NULL);
  ~TDSPBlockSolve();

  virtual Int_t GetBlockLen();
  virtual void  Print();

  TDSPSolver* GetSolver() const { return fSolver;};
  void        SetSolver(TDSPSolver*s) { fSolver = s;};

  TDSPSignal* Solve(TDSPSignal*);
  TDSPSignal* Update(TDSPSignal*);

  ClassDef(TDSPBlockSolve,1)

};


inline TDSPSignal* TDSPBlockSolve::Update(TDSPSignal *input) {
  Solve(input);
  Emit(Signal_NewData,fSignal);
  return fSignal;
}

inline Int_t TDSPBlockSolve::GetBlockLen() {
  if (fSolver) {
    return fSolver->GetNumOutputs();
  } 

  Error("GetBlockLen","No solver given !!");
  return 0;
}
#endif
