/** TDSP *******************************************************************
                          TDSPBlockfft.h  -  description
                             -------------------
    begin                : Wed Nov 14 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPBLOCKFFT_H
#define TDSPBLOCKFFT_H

#include <TDSPfft.h>
#include <TDSPBlockOperation.h>

/**the simpliest form of an STFT
 * 
 * you can build this easier by concat an 
 * TDSPChop-TDSPfft-TDSPChop, which is good for 
 * BlockLen which are greater than the signal len
 * For lower len TDSPBlockfft will be much faster
 * due to large object communication overhead 
 * in the Chop/fft/Chop-case
 *
 *@author Peter Jung
 */

class TDSPBlockfft : public TDSPfft, public TDSPBlockOperation  {

 protected:
  
  Int_t    fBlockLen;

 public: 
  TDSPBlockfft(char *name = NULL);
  ~TDSPBlockfft();

  void               SetBlockLen(Int_t b) { fBlockLen = b;};
  Int_t              GetBlockLen() { return fBlockLen; };

  // Calculate the fft/ifft of a vector
  //
  TDSPVector*  fft(TDSPVector*in, TDSPVector*out=NULL);
  TDSPVector* ifft(TDSPVector*in, TDSPVector*out=NULL);

  // Slot-Version
  //
  TDSPSignal* FFT(TDSPSignal *input);
  TDSPSignal* IFFT(TDSPSignal *input);

  ClassDef(TDSPBlockfft,1)

};

// global Blockfft-object

extern TDSPBlockfft *gBlockFFT;

// global fft-routines

TDSPVector *blockfft(TDSPVector *in, Int_t BlockLen,  TDSPVector *out = NULL);
TDSPVector *blockifft(TDSPVector *in, Int_t BlockLen, TDSPVector *out = NULL);

#endif
