/** TDSP *******************************************************************
                          TDSPClip.h  -  description
                             -------------------
    begin                : Tue Jan 22 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPCLIP_H
#define TDSPCLIP_H

#include <TDSPFunction.h>

/**A simple (Soft)-Clip Object, clips the 
   at a fixed signal amplitude and preserving
   the phase
   *@author Peter Jung
   */

class TDSPClip : public TDSPFunction  {
 protected:
  
  Double_t fClipAmplitude; // The Amplitude where to start clipping;

 public: 
  TDSPClip(char *name = NULL);
  ~TDSPClip();
  
  void     SetClip(Double_t c) { fClipAmplitude=c;};
  Double_t GetClip()           { return fClipAmplitude;};
 
  virtual TComplex F(TComplex);

  ClassDef(TDSPClip,1)

};

// Simple Clip Application. 
// Clip the Signal at fClipAmplitude and preserve the phase
//
inline TComplex TDSPClip::F(TComplex in) {
  Double_t in_abs = Abs(in);
  if (in_abs>fClipAmplitude)
    return in/in_abs*fClipAmplitude;
  else
    return in;
}

#endif
