/** TDSP *******************************************************************
                          TDSPConvolution.h  -  description
                             -------------------
    begin                : Tue Oct 30 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPCONVOLUTION_H
#define TDSPCONVOLUTION_H

#include <TDSPSignal.h>
#include <TDSPStochasticSignal.h>
#include <TDSPBlockOperation.h>
#include <TDSPOutput1.h>

/**
  *@author Peter Jung
  */

class TDSPConvolution : public TDSPOutput1, public TDSPBlockOperation {
  
 protected:

  Int_t   fBlockLen;
  Int_t   fConvOptions;

  // my Slots

  TString Slot_Response;
  TString Slot_StochasticResponse;


 public: 

  // the inputs

  TDSPVector           *ResponseFunction; ///< fixed impulse response for convolution
  TDSPStochasticSignal *StochasticResponseFunction;


  TDSPConvolution(char *name = NULL);
  ~TDSPConvolution();

  
  // Slots

  void Response(TDSPVector *r) { 
    StochasticResponseFunction = NULL;
    ResponseFunction = r;
    Emit("Response(TDSPVector*)",r);
  };          
  
  void Response(TDSPStochasticSignal *r) { 
    ResponseFunction = NULL;
    StochasticResponseFunction=r;
  };          

  void  SetOption(Option_t *opt);
  void  SetBlockLen(Int_t b) { fBlockLen = b;};
  Int_t GetBlockLen() { return fBlockLen;};
  
  TDSPVector*   Conv(TDSPVector *input);
  TDSPSignal*   Update(TDSPSignal *input);
             

  ClassDef(TDSPConvolution,1)

};

inline TDSPSignal* TDSPConvolution::Update(TDSPSignal *input) {
  this->Conv(input);
  Emit(Signal_NewData,fSignal);
  return fSignal;
}
#endif
