/** TDSP *******************************************************************
                          TDSPCorrelationFilter.h  -  description
                             -------------------
    begin                : Mon Jan 21 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPCORRELATIONFILTER_H
#define TDSPCORRELATIONFILTER_H

#include <TDSPFilter.h>

/**In the moment this calls intern 
   matlab-function from the signal
   processing toolbox
   *@author Peter Jung
  */

enum eCorrFilterMethod {
  kCorrFilterYule,
  kCorrFilterProny
};

enum eCorrFilterOptions {
  kCorrFilterNormalize = 0x1
};

class TDSPCorrelationFilter  : virtual public TDSPFilter {
  
 protected:
  eCorrFilterMethod fMethod;    // The Method
  UInt_t      fAR,fMA;          // Filter-Dimension AR=FIR, MA=IIR (=0 defaults)
  TDSPSignal *fCorrelation;     // the Signal vector containing the discrete correlation function
  UInt_t fOptions;              // Options

 public: 
  
  TDSPCorrelationFilter();
  ~TDSPCorrelationFilter();

  void             Init(Double_t MaxTime=1, 
			Int_t Entries=100);     // Initialize the Filter / Calculate the Filter
  virtual Double_t Correlation(Double_t);       // Calculates the Correlation for a given time delay
  TDSPSignal*      GetCorrelation() { return fCorrelation;};
  eCorrFilterMethod GetMethod() { return fMethod;};  // Get the Current Method
  void             SetMethod(eCorrFilterMethod m)  { fMethod = m;};  // Set the Method
  void             SetAR(UInt_t ar) { fAR = ar;};    // Set the Number of AR-Parameters
  UInt_t           GetAR() { return fAR;};           // Get the Number of AR-Parameters
  void             SetMA(UInt_t ma) { fMA = ma;};    // Set the Number of AM-Parameters
  UInt_t           GetMA() { return fMA;};           // Get the Number of AM-Parameters
  void             SetARMA(UInt_t ar, UInt_t ma);    // Set the Number of AR- and MA-Parameters

  void             EnableOption(eCorrFilterOptions o) { // Enable a option
    fOptions |= o; }
  void             DisableOption(eCorrFilterOptions o) {// Disable a option
    fOptions &= !o; }

  virtual void     Test();

  ClassDef(TDSPCorrelationFilter,1)

};


inline void TDSPCorrelationFilter::SetARMA(UInt_t ar, UInt_t ma) {
  SetAR(ar);
  SetMA(ma);
}

#endif
