/** TDSP *******************************************************************
                          TDSPFMT.h  -  description
                             -------------------
    begin                : Fri Nov 23 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef TDSPFMT_H
#define TDSPFMT_H


#include <TDSPSignal.h>
#include <TDSPFilterMap.h>

/**Implemenation of the FMT transmitter
   receiver 
   *@author Peter Jung
   */


class TDSPFMT : public TDSPFilterMap {
  
 private:

  // Some more signal-processing typical 
  // slot-names (implemented as Delegation to the
  // original slots)

  TString Delegation_Transmit;
  TString Delegation_Receive;

  TString Delegation_Transmitted;
  TString Delegation_Received;

  TString& _build_pulse_param_string(TString& start);

 protected:
  

 public: 

  // Constructor / Destructor
  //
  TDSPFMT(char *name = NULL);
  virtual ~TDSPFMT();  

  // Set/Get Parameters
  //
  UInt_t      GetNumCarriers() { return GetNumFilters(); };
  void        SetNumCarriers(Int_t n) { SetNumFilters(n);}; // Configure the filterbank for the use of "n" carriers

  TDSPMatrix* GetTransmitMatrix() { return GetMatrix();};
  TDSPMatrix* GetReceiverMatrix() { return GetInverseMatrix();};
  
  TDSPVector* GetTransmitPulse(TDSPVector *p=NULL);
  TDSPVector* GetReceiverPulse(TDSPVector *p=NULL);

  void        SetTransmitMatrix(TDSPMatrix *h) { SetMatrix(h); };
  void        SetReceiverMatrix(TDSPMatrix *h) { SetInverseMatrix(h); };

  void        SetOption(Option_t *opt);

  void        LoadPulses(char *fname, char *transmitname="TransmitMatrix", char *receivername="ReceiverMatrix");
  void        SavePulses(char *fname, char *transmitname="TransmitMatrix", char *receivername="ReceiverMatrix");
  
  TDSPSignal*        Update(TDSPSignal *input); // Calls Synthesize and Emits "Update(TDSPSignal*)"
  TDSPSignal* InverseUpdate(TDSPSignal *input); // Calls Analyse and Emits "InverseUpdate(TDSPSignal*)"

  ClassDef(TDSPFMT,1)

};
#endif
