/** TDSP *******************************************************************
                          TDSPFastBpsk.h  -  description
                             -------------------
    begin                : Mon Mai 6 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPFASTBPSK_H
#define TDSPFASTBPSK_H

#include <TDSPPsk.h>

/**Fast BPSK implementation
 *@author Peter Jung
 */

class TDSPFastBpsk : public TDSPPsk  {
 protected:

  void     GenerateSymbols(Int_t, TComplex*); // Generate a lot of symbols 
  Int_t    GenerateSymbol(TComplex *c);
  Int_t    DetectSymbol(TComplex& s);

 public: 
  TDSPFastBpsk(char *name = NULL);
  ~TDSPFastBpsk();

  ClassDef(TDSPFastBpsk,1)

};


// Generate a lot of symbols
//
inline void TDSPFastBpsk::GenerateSymbols(Int_t num,
					  TComplex *c) {
  Int_t *sym = fISymbols;
  for(register Int_t i=num;i;--i) 
    *(sym++) = GenerateSymbol(c++);
}

inline Int_t TDSPFastBpsk::GenerateSymbol(TComplex *c) {
  if (gRandom->Rndm()<.5) {
    *c      = *fSymbols;
    return 0;
  }

  *c = *(fSymbols+1);
  return 1;
}

inline Int_t TDSPFastBpsk::DetectSymbol(TComplex& s) {
  if (s.fRe<0) return 1;
  if (s.fRe>0) return 0;
  return NOT_DETECTED;
}

#endif
