/** TDSP *******************************************************************
                          TDSPFastQpsk.h  -  description
                             -------------------
    begin                : Mon Mai 6 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPFASTQPSK_H
#define TDSPFASTQPSK_H

#include <TDSPPsk.h>

/**Fast BPSK implementation
 *@author Peter Jung
 */

class TDSPFastQpsk : public TDSPPsk  {
 protected:

  void     GenerateSymbols(Int_t, TComplex*); // Generate a lot of symbols 
  Int_t    GenerateSymbol(TComplex *c);
  Int_t    DetectSymbol(TComplex& s);

 public: 
  TDSPFastQpsk(char *name = NULL);
  ~TDSPFastQpsk();

  ClassDef(TDSPFastQpsk,1)

};


// Generate a lot of symbols
//
inline void TDSPFastQpsk::GenerateSymbols(Int_t num,
					  TComplex *c) {
  Int_t *sym = fISymbols;
  for(register Int_t i=num;i;--i) 
    *(sym++) = GenerateSymbol(c++);
}

inline Int_t TDSPFastQpsk::GenerateSymbol(TComplex *c) {
  Int_t r = Int_t(gRandom->Rndm()*4.0);
  *c = *(fSymbols+r);
  return r;
}

inline Int_t TDSPFastQpsk::DetectSymbol(TComplex& s) {


  // rotiere um 45 grad entgegen Uhrzeiger
  
  //    1
  //  2   0
  //    3

  Double_t x = s.fRe-s.fIm;
  Double_t y = s.fRe+s.fIm;
  
  // 1   0
  // 2   3

  if (x<0) {
    if (y>0) return 1; 
    if (y<0) return 2;
  } 
  if (x>0) {
    if (y>0) return 0;
    if (y<0) return 3;
  } 
  return NOT_DETECTED;
}

#endif
