/** TDSP *******************************************************************
                          TDSPFilter.h  -  description
                             -------------------
    begin                : Wed Jan 9 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPFILTER_H
#define TDSPFILTER_H

#include <TDSPRating.h>
#include <TDSPSignal.h>

/**FIR/IIR-Filters
 *@author Peter Jung
  */

class TDSPFilter  : virtual public TDSPRating  {

 protected:
  
  Double_t    fNorm;            // the Norm of the filter 
  Bool_t      fNormalized;      // is the Filter already normalized ?
  
 public: 

  TDSPVector        *fFIR;
  TDSPVector        *fIIR;

  Double_t           GetNorm() const { return fNorm;};       // Return the Norm of the Filter
  void               SetNorm(Double_t n) { fNorm = n;};      // Set the Norm of the Filter
  void               Normalize();                            // Normalize the Filter
  Bool_t             IsNormalized() const { return fNormalized;}; // Is the Filter already normalized ?

  TDSPFilter();
  virtual ~TDSPFilter();

  
  TDSPVector* GetFIR() const { return fFIR; };
  TDSPVector* GetIIR() const { return fIIR; };

  void FIR(TDSPVector *fir) {fFIR = fir;fNormalized=kFALSE;};// specify the FIR Filter (vector of coefficients)
  void IIR(TDSPVector *iir) {fIIR = iir;};// specify the IIR Filter (vector of coefficients)

  TDSPVector *filter(TDSPVector *input, TDSPVector *output=NULL) { // Filter the input Signal with specified FIR/IIR-filters
    return ::filter(input,output,fFIR,fIIR);
  }

  TDSPVector *filter(TDSPFifo *input, TDSPVector *output=NULL) {   // Filter the content of the input Fifo with specified FIR/IIR-filters
    return ::filter(input,output,fFIR,fIIR);
  }

  TComplex   filter(TDSPFifo *input, TDSPFifo *output) {   // Filter the content of the input Fifo with specified FIR/IIR-filters
    return ::filter(input,output,fFIR,fIIR);
  }

  ClassDef(TDSPFilter,1)

};

#endif
