/** TDSP *******************************************************************
                          TDSPFunction.h  -  description
                             -------------------
    begin                : Tue Jan 22 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPFUNCTION_H
#define TDSPFUNCTION_H

#include <TDSPOutput1.h>

/**Base Class for pointwise manipulation 
   of TDSPSignals
   *@author Peter Jung
   */

class TDSPFunction : public TDSPOutput1  {

 protected:
  
  Double_t fInputSamplingRate; // The Sampling Rate of the actual input signal

 public: 
  TDSPFunction(char *name = NULL);
  ~TDSPFunction();

  virtual TComplex F(TComplex);
  TDSPSignal* Apply(TDSPSignal*);
  TDSPSignal* Update(TDSPSignal*);

  ClassDef(TDSPFunction,1)

};

inline TDSPSignal* TDSPFunction::Apply(TDSPSignal *input) {
  TDSPSignal *output = GetSignal();
  Int_t len=input->GetLen();
  output->SetLen(len);

  // Configure the output signal on the base
  // of the input signal
  //
  output->Configure(input);
  
  // Some Child classes would need the sampling rate 
  // of the input signal (stochastic models for example ...)
  //
  fInputSamplingRate = input->GetSamplingRate();

  TComplex   *ivec   =  input->GetVec();
  TComplex   *ovec   = output->GetVec();
  for(register Int_t i=0;i<len;i++) 
    ovec[i] = F(ivec[i]);
  
  return output;
}

inline TDSPSignal* TDSPFunction::Update(TDSPSignal *input) {

  TDSPSignal *output = Apply(input);
  Emit(Signal_NewData,output);
  return output;
}

// In this Base Class we make a identity mapping
//
inline TComplex TDSPFunction::F(TComplex in) {
  return in;
}
#endif
