/** TDSP *******************************************************************
                          TDSPJakes.h  -  description
                             -------------------
    begin                : Mon Jan 21 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPJAKES_H
#define TDSPJAKES_H

#include <TMath.h>
#include <TDSPMath.h>
#include <TDSPCorrelationFilter.h>

/**The Jakes Model for 
   mobile radio channels
   *@author Peter Jung
   */

extern const Double_t SpeedOfLight;

class TDSPJakes  : public TDSPCorrelationFilter {
  
  Double_t fSpeed;
  Double_t fDopplerFrequency;
  Double_t fCarrierFrequency;

 public: 
  TDSPJakes();
  ~TDSPJakes();

  void      SetSpeed(Double_t);            // Set the Speed in m/s
  void      SetSpeedKMH(Double_t);         // Set the Speed in km/h
  void      SetCarrierFrequency(Double_t); // Set the Carrier Frequency in Hz
  Double_t  GetDopplerFrequency();         // Return the Doppler Frequency in Hz
  void      SetDopplerFrequency(Double_t); // Set direct the Doppler Frequency and correct the speed

  Double_t  Correlation(Double_t);  // Calculates the Correlation for a given time delay

  ClassDef(TDSPJakes,1)

};

#endif
