/** TDSP *******************************************************************
                          TDSPModulation.h  -  description
                             -------------------
    begin                : Mon Mai 13 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPMODULATION_H
#define TDSPMODULATION_H

#include <TDSPBlockOperation.h>
#include <TDSPOutput1.h>

/**Modulates the input vector 
 *@author Peter Jung
 */

class TDSPModulation : public TDSPOutput1, public TDSPBlockOperation  {
 protected:

  Int_t    fBlockLen;  // Burst len

  TComplex fFrequency; // The Modulation Frequency
  Double_t fPhase;     // the initial phase
  Int_t    fBurstOff;  // the burst offset

 public: 
  TDSPModulation(char *name = NULL);
  ~TDSPModulation();

  
  void      SetFrequency(Double_t f) { fFrequency = f;};
  void      SetFrequency(TComplex &f) { fFrequency = f;};
  TComplex  GetFrequency() const { return fFrequency;};

  void      SetPhase(Double_t phase) { fPhase=phase;};
  Double_t  GetPhase() const { return fPhase;};

  virtual void  SetBlockLen(Int_t b) { fBlockLen=b;};
  virtual Int_t GetBlockLen() { return fBlockLen;};

  void      SetBurstOffset(Int_t bu) { fBurstOff=bu;};
  Int_t     GetBurstOffset() const { return fBurstOff;};

  TDSPVector* Modulate(TDSPVector*); 

  // Slots

  TDSPSignal* Update(TDSPSignal*); // Update-Slot
  void        Reset();             // Reset-Slot

  ClassDef(TDSPModulation,1)

};

inline TDSPVector* TDSPModulation::Modulate(TDSPVector *in) {
  in->Modulate(fFrequency,fBlockLen,fBurstOff, fPhase,fSignal);
  fSignal->Configure(in);
  return fSignal;
}


#endif
