/** TDSP *******************************************************************
                          TDSPOfdm.h  -  description
                             -------------------
    begin                : Fri Nov 23 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPOFDM_H
#define TDSPOFDM_H

#include <TDSPBlockfft.h>

/**Implemenation of the OFDM transmitter
   receiver 
   *@author Peter Jung
   */

class TDSPOfdm : public TDSPBlockfft  {


  Bool_t      fAutoEqualization; ///< Automatic Equalization after FFT ?
  TDSPVector *fEqualization; ///< Contains the equalization vector(for inverse update)  

 public: 
  TDSPOfdm(char *name = NULL);
  ~TDSPOfdm();

  Int_t       GetNumCarriers() { return GetBlockLen(); };
  void        SetNumCarriers(Int_t n) { SetBlockLen(n); };
  
  void        SetEqualization(TDSPVector *e);
  TDSPVector* GetEqualization() const { return fEqualization;};
  

  // Setting Options
  // opt="autoequalization" - switch on autoequalization
  
  void        SetOption(Option_t *opt);
      
  TDSPSignal* GetTransmitSignal() { return GetIFFTSignal(); };
  TDSPSignal* GetReceivedSignal() { return GetFFTSignal(); };
  Double_t    GetBandWidthEfficiency() { return Double_t(GetNumCarriers())/Double_t(GetBlockLen()+fCyclicPrefix);};
  

  virtual Bool_t      CalcEqualization(TDSPVector *r); ///< Calculate the Equalization vector from the channel response r
  virtual void        EqualizeMe(TDSPVector*in=NULL); ///< Equalize an incoming vector (if =NULL - equalize this->fFFTSignal)
  virtual TDSPSignal* Receive(TDSPSignal* in); ///< Receive and (see fAutoEqualization) Equalize 
  virtual TDSPSignal* Transmit(TDSPSignal* in); ///< Transmit 
  void                Received(TDSPSignal*);///< Emit a new received (and probably equalized) signal
  void                Transmitted(TDSPSignal*);///< Emit a new transmitted signal
  ClassDef(TDSPOfdm,1)

};

inline void TDSPOfdm::Received(TDSPSignal *r) {
  Emit("Received(TDSPSignal*)",r);
}

inline void TDSPOfdm::Transmitted(TDSPSignal *r) {
  Emit("Transmitted(TDSPSignal*)",r);
}

#endif
