/** TDSP *******************************************************************
                          TDSPOperator.h  -  description
                             -------------------
    begin                : Mon Sep 10 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPOPERATOR_H
#define TDSPOPERATOR_H

#include <TCanvas.h>
#include <TQQObject.h>
#include <TDSPData.h>
#include <TDSPSignal.h>
#include <TQQGControl.h>

/**
 *@author Peter Jung
 */

class TDSPOperator : public TQQObject {
 

 protected:


  Bool_t fInitialized; // Is this Operator already initialized ?

  void CheckInitialization() {if (!fInitialized) Init();};  // Init the Operator if not yet done already

  /////////////////////////////////////////////////
    //
  // Everything for Debugging such an Object 

  TCanvas*    fDebugWindow; ///< For Debugging ...
  TCanvas*     DebugGetWindow(char *title); ///< Opens the debug window    
  Bool_t       DebugWaitToContinue(); ///< Waiting for continue (return kFALSE to break the debug mode)

 public:
  
  TDSPOperator(char *name = NULL);
  ~TDSPOperator();
  

  // Signal Emits
  //
  virtual void Emit(const char *signal_name) {
    TQQObject::Emit(signal_name);
  }

  virtual void Emit(const char *signal_name,
		    TDSPData *f) {
    TQQObject::Emit(signal_name,(Long_t)f);
  }
  virtual void Emit(const char *signal_name,
		    TDSPVector *f) {
    TQQObject::Emit(signal_name,(Long_t)f);
  }
  virtual void Emit(const char *signal_name,
		    TDSPMatrix *f) {
    TQQObject::Emit(signal_name,(Long_t)f);
  }
  virtual void Emit(const char *signal_name,
		    TDSPSignal *f) {
    TQQObject::Emit(signal_name,(Long_t)f);
  }
   
  virtual void Emit(const char *signal_name,
		    TDSPOperator *o) {
#ifndef WIN32
    TQQObject::Emit(signal_name,(Long_t)o);
#endif
  }

  // Signals

  virtual void New(TDSPOperator*o){
    Emit("New(TDSPOperator*)",o);
  }; //*SIGNAL*
  

  virtual void Delete(TDSPOperator*o) {
    Emit("Delete(TDSPOperator*)",o);
  }; //*SIGNAL*

  virtual void Paint() {
    Emit("Paint()");
  }; //*SIGNAL*

  virtual TQQGControl* NewGControl(); ///< opens a new control 

  virtual void Reset();   //*SIGNAL*
  virtual void Init();    //*SIGNAL*

  ClassDef(TDSPOperator,1)
    
};

inline void TDSPOperator::Init() {
  fInitialized = kTRUE;
}

inline void TDSPOperator::Reset() {
  Emit("Reset()");
}


#endif
