/** TDSP *******************************************************************
                          TDSPOutput1.h  -  description
                             -------------------
    begin                : Wed Nov 14 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPOUTPUT1_H
#define TDSPOUTPUT1_H


/**Base Interface for Operators producing 
   1 output Signal
*@author Peter Jung
*/


#include <TString.h>
#include <Riostream.h>
#include "TDSPSignal.h"
#include "TDSPOperator.h"

#define kTDSPOUTPUT_CAN_OVERWRITE 0x1

class TDSPOutput1 : public TDSPOperator {

  friend istream &operator>>(istream&,TDSPOutput1&);
  friend ostream &operator<<(ostream&,TDSPOutput1&);
  friend istream &operator>>(istream&,TDSPOutput1*);
  friend ostream &operator<<(ostream&,TDSPOutput1*);


 protected:

  Bool_t      fIsActive; // Active / InActive
  UInt_t      fBits; // Some Option Bits
  
  // The Output Signal
  TDSPSignal *fSignal; 

  // The Signal to communicate to other object
  // and send them that there is data in fSignal

  TString Signal_NewData; 
  TString Slot_Update;    

 public: 

  TDSPOutput1(char *name = NULL);
  ~TDSPOutput1();

  virtual void Print();

  // Overwriting the output ?

  void   SetCanOverwrite() { fBits &= kTDSPOUTPUT_CAN_OVERWRITE;}; // Set - overwrite of the output is possible
  void   ClearCanOverwrite() { fBits &= (!kTDSPOUTPUT_CAN_OVERWRITE);}; // Clear - overwrite of the output is not possible
  Bool_t CanOverWrite() { return fBits&(!kTDSPOUTPUT_CAN_OVERWRITE);}; // overwriting of the output possible ?

  // Connect Methods
  //
  Bool_t Connect(const char *signal,
  		 TNamed     *receiver,
  		 const char *slot) { 
    return TDSPOperator::Connect(signal,receiver,slot); 
  }
  
  Bool_t Connect(TQQObject  *receiver,
		 const char *slot = NULL) {
    if (!slot) slot=Slot_Update;
    return TDSPOperator::Connect(Signal_NewData.Data(),receiver,slot);
  }
   
  Bool_t Connect(const char *signal,
		 TQQObject  *receiver,
		 const char *slot = NULL) {
    if (!slot) slot=Slot_Update;
    return TDSPOperator::Connect(signal,receiver,slot);
  }

  // This is the Standard Slot - pipes the data through

  virtual TDSPSignal* Update(TDSPSignal* input) // *SIGNAL*
    {
      fSignal->SetVec(input->GetVec());
      fSignal->SetLen(input->GetLen());
      Emit(Signal_NewData,fSignal);
      return fSignal;
    }
  void Browse(TBrowser *b = NULL);
  TDSPSignal* GetSignal() const { return fSignal;};
  void        SetSignal(TDSPSignal *s) { fSignal = s;};
  
  void        Activate() { fIsActive = kTRUE;}; // *MENU*
  void        Deactivate() { fIsActive = kFALSE;};  // *MENU*
  
  ClassDef(TDSPOutput1,1)
  
};

#endif
