/** TDSP *******************************************************************
                          TDSPSignal.h  -  description
                             -------------------
    begin                : Thu Nov 1 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPSIGNAL_H
#define TDSPSIGNAL_H

#include <TDSPMatrix.h>
#include <TDSPFifo.h>

class TDSPFifo;

/**a discrete complex signal
  *@author Peter Jung
  */


class TDSPSignal : public TDSPMatrix {
  

 protected:
  
  TString fName; //signal name
  TString fTitle; // signal title

 public: 

  TDSPSignal(Int_t len=0);
  TDSPSignal(Int_t rows, Int_t cols);
  TDSPSignal(char *name);
  virtual ~TDSPSignal();


  // Friend functions

  friend Bool_t      operator==(TDSPSignal &, TDSPSignal &);
  friend istream    &operator>>(istream&,TDSPSignal&);
  friend ostream    &operator<<(ostream&,TDSPSignal&);
  friend istream    &operator>>(istream&,TDSPSignal*);
  friend ostream    &operator<<(ostream&,TDSPSignal*);

  
/*   void        ModulateMe(TComplex, Int_t b=0,Double_t p=0);                       // Modulate the signal (multiply with a frequency) */
/*   TDSPSignal* Modulate(TComplex, TDSPSignal* = NULL,Int_t b=0,Double_t p=0);      // Modulate the signal  */

  virtual void     SetName(const char *name); // *MENU*
  virtual void     SetNameTitle(const char *name, const char *title);
  virtual void     SetTitle(const char *title=""); // *MENU*
  virtual void     ls(Option_t *option="") const;
  virtual void     Print(Option_t *option="");
  virtual const char  *GetName() const {return fName.Data();}
  virtual const char  *GetTitle() const {return fTitle.Data();}
  
  

  TDSPSignal &operator=(TDSPFifo&); 
  TDSPSignal &operator=(TDSPFifo*f) { return operator=(*f);}; 

  ClassDef(TDSPSignal,1)
    
};
  
inline Bool_t operator==(TDSPSignal & z1, TDSPSignal & z2) {
  if (z1.fSamplingRate!=z2.fSamplingRate) return kFALSE;
  return operator==(*static_cast<TDSPVector*>(&z1),
		    *static_cast<TDSPVector*>(&z1));
  
}

/* inline void TDSPSignal::ModulateMe(TComplex  freq, Int_t b,Double_t p) { */
/*   TDSPVector::ModulateMe(freq,b); */
/* } */
/* inline TDSPSignal* TDSPSignal::Modulate(TComplex  freq, TDSPSignal *out, Int_t b, Double_t p) { */
/*   return static_cast<TDSPSignal*>(TDSPVector::Modulate(freq,out,b,p)); */
/* } */
#endif
