/** TDSP *******************************************************************
                          TDSPSignalStudy.h  -  description
                             -------------------
    begin                : Die Okt 8 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPSIGNALSTUDY_H
#define TDSPSIGNALSTUDY_H

#include <TDSPOutput1.h>
#include <TDSPBlockOperation.h>

/**Studies several properties of a incoming signal
 *@author Peter Jung
 */


class TDSPSignalStudy : public TDSPOutput1, public TDSPBlockOperation  {

 private:

  UInt_t fWhat; // What to do ?

  Int_t fBlockLen;

  // Settings for Crestfactor
  
  TDSPVector *fMean; //!
  TDSPVector *fMaxAbs; //!
  TDSPVector *fCrest; //!
  

  
 public: 

  TDSPSignalStudy(char *name = NULL);
  ~TDSPSignalStudy();

  // The Analyse
  virtual void Analyse(TDSPSignal* input);

  // Settings

  Int_t         GetBlockLen() {return fBlockLen; };
  void          SetBlockLen(Int_t bl) { fBlockLen=bl;}; 

  void          EnableCrestFactor() { fWhat|=0x1;}; 
  void          DisableCrestFactor() { fWhat &= !0x1; };
  Bool_t        IsCrestFactor() {return fWhat&0x1;};
  TDSPVector*   GetMean() const { return fMean;};
  TDSPVector*   GetMaxAbs() const { return fMaxAbs;};
  TDSPVector*   GetCrestFactor() const { return fCrest;};

  // The Slot - analyse and pipe the input signal through

  TDSPSignal* Update(TDSPSignal *input) {
    Analyse(input);
    Emit(Slot_Update,input);
    return input;
  }

  ClassDef(TDSPSignalStudy,1)

};

#endif
