/** TDSP *******************************************************************
                          TDSPStft.h  -  description
                             -------------------
    begin                : Wed Nov 14 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPSTFT_H
#define TDSPSTFT_H

#include <TDSPfft.h>

/**the STFT
 *
 * This is the interface to the STFT. 4 Slots are avaiable
 * Connect a the "atom"/"Window function" with "Window(TDSPSignal*)". Analog
 * connect the dual with "DualWindow(TDSPSignal*)". With the Slots
 * "FFT(TDSPSignal*)" the Fourier transformation is calculated with the Dual
 * Window. With "IFFT(TDSPSignal*)" the Inverse Fourier Transformation is 
 * calculated according to the Window itself
 *
 *@author Peter Jung
 */

class TDSPStft : public TDSPfft  {

 protected:

  TDSPSignal   *fSTFT_Signal;
  TDSPSignal   *fISTFT_Signal;
  
  TDSPSignal   *fWindow;      // The Window-Vector
  TDSPSignal   *fDualWindow;  // The Dual-Window-Vector

  Int_t    fMulti;
  Int_t    fBlockLen;

 public: 
  TDSPStft(char *name);
  ~TDSPStft();

  // Set/Get the Blocklen (in NOFDM thats is the Number of
  // carriers Nc )

  void   SetBlockLen(Int_t b) { fBlockLen = b; };
  Int_t  GetBlockLen() { return fBlockLen; };

  // Set/Get the Multiplikator Lambda
  // This defines in NOFDM the symbol length N0 = Nc/Lambda
  // and the frequency spacing M0 = TF/N0 because TF is
  // defined as TF=Ng/Lambda and Ng is the number samples of the
  // window atom function g()
  //
  void   SetMultiplikator(Int_t b) { fMulti = b;};
  Int_t  GetMultiplikator() { return fMulti; };


  // The Slots

  // Connect a window function
  //
  void Window(TDSPSignal *input) {fWindow = input;};

  // Connect the dual window function
  //
  void DualWindow(TDSPSignal *input) {fDualWindow = input;};

#ifdef HUHU
  TDSPSignal* FFT(TDSPSignal *input);
  TDSPSignal* IFFT(TDSPSignal *input);
#endif

  ClassDef(TDSPStft,1)

};

#endif
