/** TDSP *******************************************************************
                          TDSPStochasticSignal.h  -  description
                             -------------------
    begin                : Mon Dec 10 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPSTOCHASTICSIGNAL_H
#define TDSPSTOCHASTICSIGNAL_H

#include <TDSPStatistics.h>
#include <TDSPSignal.h>
#include <TDSPBlockFifoFilter.h>

/**Stochastic Signal which has a TDSPSignal
   Representation
   *@author Peter Jung
   */


class TDSPStochasticSignal : public TDSPSignal, public TDSPStochasticProcess  {
  
 protected:
  
  TDSPSignal          *fARMAInput;  //  the place for the ARMA Filter input
  TDSPBlockFifoFilter *fFifoFilter; //  a Filter to produce correlated Stochastics
  
  Int_t  fAllocedNumMeans;    
  
  Int_t    fRepresentation;                // the actual representation

  Bool_t    fVecChanged;                // Does fVec really has been changed (or its size) ?


 public: 

  // Constructor and Destructor
  //
  TDSPStochasticSignal(Int_t len = 0);
  virtual ~TDSPStochasticSignal();
  
  // Time depended convolution
  //
  void      Conv(TDSPVector* input,  
		 TDSPVector* output,  
		 Int_t       opt = kTDSPVector_Conv_Defaults); // Convolution 


  
  void     SetAsModel();   // Switch the Meaning of fVec and fModel
  void     SetFifoFilter(TDSPBlockFifoFilter *f) {fFifoFilter=f;};     // Set a FifoFilter 
  void     ChangeRepresentation();

  virtual void    Reset();

  ClassDef(TDSPStochasticSignal,1)

};


inline void TDSPStochasticSignal::ChangeRepresentation() {
  
  switch(fMode) {
    
  case kGauss:     // a complex gauss on all taps
    RandNC(fMeans,fSigmas[0]);
    break;

  case kRayleigh:     // a real Rayleigh on all taps
    RandRayleigh(fSigmas[0].fRe);
    break;
    
  case kFilterGauss:   // a filtered gauss
    {
      Double_t  filternorm = fFifoFilter->GetFilter()->GetNorm();

      fARMAInput->SetLen(Num);
      fARMAInput->RandNC(fMeans,fSigmas[0]/filternorm/filternorm);

      if (fFifoFilter) {
	fFifoFilter->filter(fARMAInput,this);	
      }
    }
    break;
    
  case kNoChange:  // Nothing to do
    for(register Int_t i=0;i<Num;i++) 
      fVec[i] = fMeans[i];
    break;
  }  
}
  
#endif
