/** TDSP *******************************************************************
                          TDSPWienerProcess.h  -  description
                             -------------------
    begin                : Die Mai 21 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPWIENERPROCESS_H
#define TDSPWIENERPROCESS_H

#include "TDSPOutput1.h"

/**Implementation of the Wiener-Levi-Process
 *@author Peter Jung
 */

class TDSPWienerProcess : public TDSPOutput1  {
protected:
  Double_t fVariance;   // the Variance
  Double_t fBurstlen;   // 
  Double_t fBurstOff;   //
  Double_t fPhase;      // Start Phase

 public: 
  TDSPWienerProcess(char *name=NULL);
  ~TDSPWienerProcess();


  static TDSPVector* MultRandPhase(TDSPVector* input,
				   Double_t    B=1,
				   Double_t    burstlen=0,
				   Double_t    burstoff=0,
				   Double_t    Phase=0,
				   TDSPVector* output = NULL 
				   );  // init with rand. wiener phase noise mult. on an input v
  
  virtual     TDSPVector* Apply(TDSPVector* input, TDSPVector *output=NULL);
  virtual     TDSPSignal* Update(TDSPSignal* input);

  void        SetVariance(Double_t v) { fVariance=v;};
  Double_t    GetVariance() const {return fVariance;};
  void        SetBurstLen(Double_t r) { fBurstlen=r;};
  Double_t    GetBurstLen() const { return fBurstlen;};
  
  void        SetPhase(Double_t p) { fPhase=p;};
  Double_t    GetPhase() const { return fPhase;};
  
  void        SetBurstOffset(Double_t o) {fBurstOff=o;};
  Double_t    GetBurstOffset() const { return fBurstOff;};
  
  ClassDef(TDSPWienerProcess,1)

};

#endif
