/** TDSP *******************************************************************
                          MatrixRoutines.h  -  description
                             -------------------
    begin                : Mon Feb 18 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MATRIXROUTINES_H
#define MATRIXROUTINES_H

///Global Routines and matlab-equivalents

inline TDSPMatrix* Transform(TDSPMatrix *A, TDSPMatrix* X, TDSPMatrix *result=NULL) { return A->Transform(X,result);};
inline TDSPMatrix* AdjointTransform(TDSPMatrix *A, TDSPMatrix* X, TDSPMatrix *result=NULL) { return A->AdjointTransform(X,result);};
TDSPVector*        Affine(TDSPMatrix *A, TDSPVector *x, TDSPVector *b=NULL, TDSPVector *result=NULL);
TDSPMatrix*        UnitMatrix(Int_t nrows, Int_t ncols=0, TComplex diag=1.0);
TDSPMatrix*        FlatMatrix(Int_t nrows, Int_t ncols=0, TComplex fil=1.0);
inline TDSPMatrix* ZeroMatrix(Int_t nrows, Int_t ncols=0) {return FlatMatrix(nrows,ncols,0.0);}
inline TDSPMatrix* Zeros(Int_t nrows, Int_t ncols=0) {return ZeroMatrix(nrows,ncols);};
inline TDSPMatrix* Ones(Int_t nrows, Int_t ncols=0) {return FlatMatrix(nrows,ncols,1.0);}
inline TComplex    Det(TDSPMatrix *a) { return a->Det();};

TDSPMatrix*        RowStack(   TDSPVector*, Int_t n, TDSPMatrix *result=NULL); ///stack the vector as rows of a matrix
TDSPMatrix*        ColumnStack(TDSPVector*, Int_t n, TDSPMatrix *result=NULL); ///stack the vector as columns of a matrix
TDSPMatrix*        ConvMatrix( TDSPVector*, Int_t m=0, Int_t n=0, TDSPMatrix *result=NULL,Bool_t cyclic=kTRUE,Int_t Offset=0); 
TDSPMatrix*        ConvMatrix( TDSPMatrix*, Int_t m=0, Int_t n=0, TDSPMatrix *result=NULL,
			       Bool_t cyclic=kTRUE,Int_t mOffset=0,Int_t nOffset=0); 
TDSPMatrix*        Toeplitz(   TDSPVector*, Int_t m=0, Int_t n=0, TDSPMatrix *result=NULL,Bool_t cyclic=kTRUE); 
TDSPMatrix*        RandNC(UInt_t M, UInt_t N, Double_t sigma=1.);
#endif
