/** TDSP *******************************************************************
                          TDSPBlockOperation.h  -  description
                             -------------------
    begin                : Don Mai 16 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPBLOCKOPERATION_H
#define TDSPBLOCKOPERATION_H

#include <Rtypes.h>
#include <TDSPVector.h>

/**Some base, fundamental operations for Blocks - abstract class !! 
 *@author Peter Jung
 */

class TDSPBlockOperation {
 public: 
  TDSPBlockOperation();

  virtual ~TDSPBlockOperation();
  virtual Int_t      GetBlockLen() = 0;
  virtual UInt_t     TrimBlock(UInt_t l) { return (l/GetBlockLen())*GetBlockLen();}; ///< Trim a blocklen to the desired len
  static TDSPVector *Sum(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        Sum(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *SumDiff(Int_t, TDSPVector *in1, 
			     TDSPVector *in2,TDSPVector*result=NULL, 
			     TComplex scaler=1); ///< sum of the differences
  TDSPVector*        SumDiff(TDSPVector *in1, TDSPVector *in2, 
			     TDSPVector*result=NULL, TComplex scaler=1);  ///< sum of the differences

  static TDSPVector *MeanDiff(Int_t, TDSPVector *in1, 
			      TDSPVector *in2,TDSPVector*result=NULL, 
			      TComplex scaler=1);///< mean of the differences
  TDSPVector*        MeanDiff(TDSPVector *in1, TDSPVector *in2, 
			      TDSPVector*result=NULL, TComplex scaler=1);  ///< mean of the differences

  static TDSPVector *Sum2Diff(Int_t, TDSPVector *in1, TDSPVector *in2,TDSPVector*result=NULL, TComplex scaler=1);
  TDSPVector*        Sum2Diff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1); 

  static TDSPVector *Mean2Diff(Int_t, TDSPVector *in1, 
			       TDSPVector *in2,TDSPVector*result=NULL, 
			       TComplex scaler=1);///< mean of the differences
  TDSPVector*        Mean2Diff(TDSPVector *in1, TDSPVector *in2, 
			       TDSPVector*result=NULL, TComplex scaler=1);  ///< mean of the differences
      
  static TDSPVector *Mean(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        Mean(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MeanAbs2(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        MeanAbs2(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *I_MeanAbs2(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        I_MeanAbs2(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MeanAbs(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        MeanAbs(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *I_MeanAbs(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        I_MeanAbs(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MaxAbs2(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        MaxAbs2(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *I_MaxAbs2(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        I_MaxAbs2(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MinAbs2(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        MinAbs2(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *I_MinAbs2(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        I_MinAbs2(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MaxAbs(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        MaxAbs(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *I_MaxAbs(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        I_MaxAbs(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MinAbs(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        MinAbs(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *I_MinAbs(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        I_MinAbs(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MeanPower(Int_t, TDSPVector *in, TDSPVector*result=NULL);
  TDSPVector*        MeanPower(TDSPVector *in, TDSPVector*result=NULL);

  static TDSPVector *MSE(Int_t, TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);
  TDSPVector*        MSE(TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);

  static TDSPVector *SumPhaseDiff(Int_t, TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);
  TDSPVector*        SumPhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);

  static TDSPVector *Sum2PhaseDiff(Int_t, TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);
  TDSPVector*        Sum2PhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);

  static TDSPVector *MeanPhaseDiff(Int_t, TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);
  TDSPVector*        MeanPhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);

  static TDSPVector *Mean2PhaseDiff(Int_t, TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);
  TDSPVector*        Mean2PhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result=NULL, TComplex scaler=1);

  ClassDef(TDSPBlockOperation,1)

};

inline TDSPVector* TDSPBlockOperation::Sum(TDSPVector *in, TDSPVector *result) {
  return Sum(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::Mean(TDSPVector *in, TDSPVector *result) {
  return Mean(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::MeanAbs2(TDSPVector *in, TDSPVector *result) {
  return MeanAbs2(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::I_MeanAbs2(TDSPVector *in, TDSPVector *result) {
  return I_MeanAbs2(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::MeanAbs(TDSPVector *in, TDSPVector *result) {
  return MeanAbs(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::I_MeanAbs(TDSPVector *in, TDSPVector *result) {
  return I_MeanAbs(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::MaxAbs2(TDSPVector *in, TDSPVector *result) {
  return MaxAbs2(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::I_MaxAbs2(TDSPVector *in, TDSPVector *result) {
  return I_MaxAbs2(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::MinAbs2(TDSPVector *in, TDSPVector *result) {
  return MinAbs2(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::I_MinAbs2(TDSPVector *in, TDSPVector *result) {
  return I_MinAbs2(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::MaxAbs(TDSPVector *in, TDSPVector *result) {
  return MaxAbs(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::I_MaxAbs(TDSPVector *in, TDSPVector *result) {
  return I_MaxAbs(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::MinAbs(TDSPVector *in, TDSPVector *result) {
  return MinAbs(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::I_MinAbs(TDSPVector *in, TDSPVector *result) {
  return MinAbs(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::MeanPower(TDSPVector *in, TDSPVector *result) {
  return MeanPower(GetBlockLen(), in, result);
}

inline TDSPVector* TDSPBlockOperation::SumDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return SumDiff(GetBlockLen(),in1,in2,result,scaler);
}  
inline TDSPVector* TDSPBlockOperation::MeanDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return MeanDiff(GetBlockLen(),in1,in2,result,scaler);
}  
inline TDSPVector* TDSPBlockOperation::Sum2Diff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return Sum2Diff(GetBlockLen(),in1,in2,result,scaler);
}  
inline TDSPVector* TDSPBlockOperation::Mean2Diff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return Sum2Diff(GetBlockLen(),in1,in2,result,scaler);
}  
inline TDSPVector* TDSPBlockOperation::MSE(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return Mean2Diff(GetBlockLen(),in1,in2,result,scaler);
}  

inline TDSPVector* TDSPBlockOperation::MSE(Int_t bl,TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return Mean2Diff(bl,in1,in2,result,scaler);
}  

inline TDSPVector* TDSPBlockOperation::SumPhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return SumPhaseDiff(GetBlockLen(),in1,in2,result,scaler);
}  

inline TDSPVector* TDSPBlockOperation::Sum2PhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return Sum2PhaseDiff(GetBlockLen(),in1,in2,result,scaler);
}  

inline TDSPVector* TDSPBlockOperation::MeanPhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return MeanPhaseDiff(GetBlockLen(),in1,in2,result,scaler);
}  
inline TDSPVector* TDSPBlockOperation::Mean2PhaseDiff(TDSPVector *in1, TDSPVector *in2, TDSPVector*result, TComplex scaler) {
  return Mean2PhaseDiff(GetBlockLen(),in1,in2,result,scaler);
}  
#endif
