/** TDSP *******************************************************************
                          TDSPSolver.h  -  description
                             -------------------
    begin                : Thu Mar 14 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TDSPSOLVER_H
#define TDSPSOLVER_H

#include <TQQObject.h>
#include <TDSPVector.h>

/**Base class for solvers
  *@author Peter Jung
  */

class TDSPSolver : public TQQObject {

 protected:

  Bool_t fHaveSolution; ///< Is there a solution ?
  
  TDSPVector* fY; //! Vector y in the equation y=A(x)+b
  TDSPVector* fX; //! Solution Vector x in the equation y=A(x)+b
  Int_t       fNumXAlphabet;
  TComplex*   fXAlphabet;
  Double_t    fMinDist; ///< Minimal Distances for break the ML-Searching

 public: 
  TDSPSolver();
  virtual ~TDSPSolver();

  virtual Int_t GetNumInputs() = 0;
  virtual Int_t GetNumOutputs() = 0;
  virtual Double_t Redundancy() { return Double_t(GetNumOutputs())/Double_t(GetNumInputs());};

  virtual Bool_t Solve() = 0; ///< Solve - Return kTRUE if there are solutions 

  void SetY(TDSPVector*y) { fY = y;};   ///< Set the Vector y of the equation y=A(x)+b
  void SetY(TDSPVector&y) { SetY(&y);}; ///< Set the Vector y of the equation y=A(x)+b
  void SetX(TDSPVector*x) { fX = x;};   ///< Set the Vector x (the place for the solution) of the equation y=A(x)+b
  void SetX(TDSPVector&x) { SetX(&x);}; ///< Set the Vector x (the place for the solution) of the equation y=A(x)+b
  void SetXAlphabet(TDSPVector*a) { SetXAlphabet(a->GetVec(),a->GetLen());};
  void SetXAlphabet(TComplex*a, Int_t l); ///< Set the possible complex values for each X_i
  void SetXMinDist(Double_t); ///< Set the minimal distance for breaking the ML-Algorithm
  
  TDSPVector* Y() const    { return fY; }; ///< Get the Vector y of the equation y=A(x)+b
  TDSPVector* GetY() const { return fY; }; ///< Get the Vector y of the equation y=A(x)+b
  TDSPVector* X() const    { return fX; }; ///< Get the Vector x (the solution) of the equation y=A(x)+b
  TDSPVector* GetX() const { return fX; }; ///< Get the Vector x (the solution) of the equation y=A(x)+b
  
  virtual void Print() = 0;

  ClassDef(TDSPSolver,1)

};

#endif
