/** TDSP *******************************************************************
                    VectorRoutines.h  -  description
                             -------------------
    begin                : Mon Feb 18 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VECTORROUTINES_H
#define VECTORROUTINES_H

inline Double_t Var(TDSPVector *s1, TDSPVector *weights=NULL)  ///< Calculate the Variance of s1 weighted by weights
{
  return s1->Variance(weights);
}

inline Double_t Var(TDSPVector &s1) ///< Calculate the Variance of s1
{
  return s1.Variance();
}

inline Double_t Std(TDSPVector *s1) ///< Calculate the Standard Deviation of s1 (normalized N-1)
{
  return TMath::Sqrt(s1->Variance()/(s1->GetLen()-1.));
}

inline Double_t Std(TDSPVector& s1) ///< Calculate the Standard Deviation of s1 (normalized N-1)
{
  return Std(&s1);
}

inline Double_t Dist(TDSPVector *s1, TDSPVector *s2) {
  return s1->Dist(s2);
}
inline Double_t Dist2(TDSPVector *s1, TDSPVector *s2) {
  return s1->Dist2(s2);
}
inline TComplex Dot(TDSPVector *s1, TDSPVector *s2, Bool_t conj=kTRUE) {
  return s1->Dot(s2,conj);
}
inline Double_t Dot2(TDSPVector *s1, TDSPVector *s2) {
  return s1->Dot2(s2);
}

inline Double_t Dist(TDSPVector& s1, TDSPVector& s2) {
  return s1.Dist(s2);
}
inline Double_t Dist2(TDSPVector& s1, TDSPVector& s2) {
  return s1.Dist2(s2);
}
inline TComplex Dot(TDSPVector& s1, TDSPVector& s2, Bool_t conj=kTRUE) {
  return s1.Dot(s2,conj);
}
inline Double_t Dot2(TDSPVector& s1, TDSPVector& s2) {
  return s1.Dot2(s2);
}

TDSPVector* Index(TComplex x, TComplex y=0, TComplex step=1, TDSPVector *result=NULL);

inline TDSPVector* Conv(TDSPVector *in, TDSPVector *h, TDSPVector *out=NULL, 
			Int_t opt=kTDSPVector_Conv_Defaults, Int_t bl=0) {
  return h->Conv(in,out,opt,bl);
}

TDSPVector* Rect(Int_t L, Int_t N, Double_t norm=1,TDSPVector *in=NULL); ///Rectangular pulse
TDSPVector* Rect_FFTShift(Int_t L, Int_t N, Double_t norm=1,TDSPVector *in=NULL);  ///Rectangular pulse + fftshift
TDSPVector* Gaussnk(Int_t L,Double_t norm=1,TDSPVector *in=NULL); ///Gaussnk - Fourier invariant Gaussian
TDSPVector* Gaussnk_FFTShift(Int_t L,Double_t norm=1,TDSPVector *in=NULL); ///Gaussnk - Fourier invariant Gaussian + fftshift
TDSPVector* Rcos(Int_t L, Int_t N, Double_t rolloff, Double_t norm=1.,TDSPVector *output=NULL); ///< Raised Cosine 
TDSPVector* Rcos_FFTShift(Int_t L, Int_t N, Double_t rolloff, Double_t norm=1., TDSPVector *output=NULL); ///< Raised Cosine + fftshift
TDSPVector* RandNC(UInt_t L, Double_t sigma=1.);
TDSPVector* Delta(UInt_t L, Int_t one_pos=0.); ///< Vector of length L with "1" at one_pos and the rest is zero
#endif
