/** TDSP *******************************************************************
                          dl_atlas.h  -  description
                             -------------------
    begin                : Thu Nov 1 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _DL_ATLAS_H
#define _DL_ATLAS_H


#ifndef WIN32
# define DL_ATLAS 1
#endif

#ifndef CBLAS_ENUM_DEFINED_H
#define CBLAS_ENUM_DEFINED_H
enum CBLAS_ORDER {CblasRowMajor=101, CblasColMajor=102 };
enum CBLAS_TRANSPOSE {CblasNoTrans=111, CblasTrans=112, CblasConjTrans=113,
		      AtlasConj=114};
enum CBLAS_UPLO  {CblasUpper=121, CblasLower=122};
enum CBLAS_DIAG  {CblasNonUnit=131, CblasUnit=132};
enum CBLAS_SIDE  {CblasLeft=141, CblasRight=142};
#endif


typedef void  (*dl_cblas_zgemm_function)(CBLAS_ORDER,CBLAS_TRANSPOSE,CBLAS_TRANSPOSE,
					 const int, const int, const int, const void*,
					 const void*,const int,const void*,const int,
					 const void*,void*,const int);

typedef void  (*dl_cblas_cgemm_function)(CBLAS_ORDER,CBLAS_TRANSPOSE,CBLAS_TRANSPOSE,
					 const int, const int, const int, const void*,
					 const void*,const int,const void*,const int,
					 const void*,void*,const int);

typedef void  (*dl_cblas_zgemv_function)(CBLAS_ORDER,CBLAS_TRANSPOSE,
					 const int, const int, const void*,
					 const void*,const int,const void*,const int,
					 const void*,void*,const int);

typedef void  (*dl_cblas_cgemv_function)(CBLAS_ORDER,CBLAS_TRANSPOSE,
					 const int, const int, const void*,
					 const void*,const int,const void*,const int,
					 const void*,void*,const int);

typedef void (*dl_lapack_zgesdd_function)(char*,    // JOB U
					  int*,    int*,      // M , N
					  void*,   int*,  // A, LDA
					  void*,        // S (singular values)
					  void*,   int*,  // U
					  void*,   int*,  // V^*
					  void*,   int*, // WORK,LWORK
					  double*,         // RWORK
					  int*,         // IWORK
					  int*             // INFO
					  );

typedef void (*dl_lapack_cgesvd_function)(char*, char*,  // JOB U, JOB_V
					  int*, int*,      // M , N
					  void*, int*,  // A, LDA
					  void*,        // S (singular values)
					  void*, int*,  // U
					  void*, int*,  // V^*,
					  void*, int*,  // WORK,LWORK
					  float*,        // RWORK
					  int*,         // IWORK
					  int*             // INFO
					  );
typedef void (*dl_lapack_zgesvd_function)(char*, char*,   // JOB U, JOB_V
					  int*,    int*,      // M , N
					  void*,   int*,  // A, LDA
					  void*,        // S (singular values)
					  void*,   int*,  // U
					  void*,   int*,  // V^*
					  void*,   int*, // WORK,LWORK
					  double*,         // RWORK
					  int*             // INFO
					  );

typedef void (*dl_lapack_cgesdd_function)(char*,   // JOB U, 
					  int*, int*,      // M , N
					  void*, int*,  // A, LDA
					  void*,        // S (singular values)
					  void*, int*,  // U
					  void*, int*,  // V^*,
					  void*, int*,  // WORK,LWORK
					  float*,        // RWORK
					  int*             // INFO
					  );

extern dl_cblas_zgemm_function dl_cblas_zgemm;
extern dl_cblas_cgemm_function dl_cblas_cgemm;
extern dl_cblas_zgemv_function dl_cblas_zgemv;
extern dl_cblas_cgemv_function dl_cblas_cgemv;
extern dl_lapack_zgesdd_function dl_lapack_zgesdd;
extern dl_lapack_cgesdd_function dl_lapack_cgesdd;
extern dl_lapack_zgesvd_function dl_lapack_zgesvd;
extern dl_lapack_cgesvd_function dl_lapack_cgesvd;

#endif
