/** TDSP *******************************************************************
                          TmxArray.h  -  description
                             -------------------
    begin                : Tue Jan 8 2002
    copyright            : (C) 2002 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TMXARRAY_H
#define TMXARRAY_H

#include <TNamed.h>
#include <TDSPVector.h>
#include <TDSPMatrix.h>

/**C++-Wrapper Class for Mathworks 
   mxArrray structure
   *@author Peter Jung
   */

class TmxArray : public TNamed  {

 protected:
  
  Bool_t  fMymx;    //  is that my array ?
  void   *fmxArray; //!  

 public: 
  
  TmxArray(); 
  TmxArray(const char*);
  TmxArray(void*);
  
  ~TmxArray();
    
  void        Set(TDSPVector*);
  void        Set(TDSPMatrix*);
  TmxArray&   operator=(TDSPVector& x) {Set(&x);return *this;};
  TmxArray&   operator=(TDSPMatrix& x) {Set(&x);return *this;};

  void*       Getmx() { return fmxArray;};
  void        Setmx(void* m);
  TDSPMatrix* Get(TDSPMatrix *s);
  TDSPVector* Get(TDSPVector *s = NULL);
  Bool_t      IsString();
  char*       GetString(char *buf = NULL, Int_t buflen=0);
  TDSPVector* GetRow(Int_t r, TDSPVector*row=NULL);
  TDSPVector* GetColumn(Int_t r, TDSPVector*row=NULL);
  Int_t       GetN();
  Int_t       GetM();
  Int_t       GetCols() { return GetN();};
  Int_t       GetRows() { return GetM();};
  void        SetName(const char *);
  const char* GetName();
  void        Print();
  void        Clean(); ///< Cleanup - remove the fmxarray if alloced by me 
  void        Detach(); ///< Detach the mxArray - is necessary for use in return arguments of mex-files
  /////////////////////////////////////////////////////
  //
  // some operators
  //
  TComplex    Element(Int_t index);
  TComplex    operator()(Int_t index) { return Element(index);};
        
  ClassDef(TmxArray,1)

};

#endif
