/** TDSP *******************************************************************
                          TQQObject.h  -  description
                             -------------------
    begin                : Fri Nov 2 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TQQOBJECT_H
#define TQQOBJECT_H


#include <TNamed.h>
#include <TBrowser.h>
#include <TQConnection.h>
#include <TQObject.h>

/**This implements some more signal/slot
functionality to TQObjSender, like signal/slot browsing
  *@author Peter Jung
  */


class TQQObject;
class TQQGControl;

class TQQDelegation : public TNamed {

 public:
  TList* senders;
  TList* receivers;

  TQQDelegation()  { senders=NULL;receivers=NULL; };
  ~TQQDelegation() {
    if (senders)   delete senders;
    if (receivers) delete receivers;
  }

  ClassDef(TQQDelegation,1)

};

class TQQDelegated_Receiver : public TNamed {
 public:
  char       *receiver_class; //!
  TQQObject  *receiver;
  char       *slot; //!
  char       *ToString();
  TQQDelegated_Receiver(){};
  ~TQQDelegated_Receiver(){};

  ClassDef(TQQDelegated_Receiver,1)

};

class TQQDelegated_Sender : public TNamed {
 public:
  char       *signal; //!
  TQQObject  *sender;
  char       *ToString();

  TQQDelegated_Sender(){};
  ~TQQDelegated_Sender(){};

  ClassDef(TQQDelegated_Sender,1)

};

class TQQObject : public TNamed , public TQObjSender  {


  TList *fListOfSlots;  // List of all Slot names 

  TList *fListOfDelegations;   // List of Delegations (somewhat between Signals and Slots)

 

 public: 
  TQQObject();
  virtual ~TQQObject();
  void PrintSignals();
  void PrintSlots();
  void PrintDelegations();

  char*         ConnectionToString(char *signal, TQConnection*);
  TQConnection* StringToConnection(char *signal, char*);

  TQQDelegation* FindDelegation(const char *name);
  TList*         FindSignal(const char *name);
  TList*         FindSlot(const char *name);
  
  TList *GetListOfSlots()       { return fListOfSlots; };
  TList *GetListOfSignals()     { return fListOfSignals; };
  TList *GetListOfDelegations() { return fListOfDelegations; };
 
  char* AddSignal(char*);
  char* AddSignals(Int_t num, ...);

  char* AddSlot(char*);
  char* AddSlots(Int_t num, ...);

  char* AddDelegation(char *);
  char* AddDelegations(Int_t num, ...);

  char* RenameDelegation(char* name,char* newname);

  virtual void   RegisterSignalsAndSlots() {};
  virtual Bool_t IsFolder() const { return kTRUE;};

  // Simplier Connect on TNamed-Object (they now their classnames...)
  //
  Bool_t Connect(const char *signal,
		 TNamed     *receiver,
		 const char *slot) {
    
    return Connect(signal,receiver->ClassName(), 
		   receiver, slot);
    
  }

  Bool_t Connect(const char *signal,
		 const char *receiver_class,
		 void       *receiver,
		 const char *slot);
  
  static Bool_t  Connect(const char *sender_class,
			 const char *signal,
			 const char *receiver_class,
			 void *receiver,
			 const char *slot);
  
  
  void Browse(TBrowser *b = NULL);

  // Signal to Update external components (usally controls)
  //
  virtual void   Update() {
    Emit("Update()");
  } //*SIGNAL*
  
  
  virtual TQQGControl* NewGControl(); ///< opens a new control 
  virtual void Edit();    //*MENU*

  ClassDef(TQQObject,1)

};




#endif
