/** TDSP *******************************************************************
                          TQQObjectGControl.h  -  description
                             -------------------
    begin                : Tue Nov 6 2001
    copyright            : (C) 2001 by Peter Jung
    email                : jung@hhi.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TQQOBJECTGCONTROL_H
#define TQQOBJECTGCONTROL_H

#include <TGListTree.h>
#include <TGTab.h>
#include <TGWindow.h>
#include <TGListTree.h>
#include <TGButton.h>
#include <TGCanvas.h>
#include <TGMenu.h>
#include <TQQGControl.h>
#include <TQQObjectControl.h>
#include <TGControlList.h>
#include <TGTabDialog.h>

/**Graphical Control of an operator
  *@author Peter Jung
  */


#define kAddDelegation  1000
#define kEditDelegation 1001

#define kAddSignal      2000
#define kEditSignal     2001
#define kEmitSignal     2002

#define kAddSlot        3000
#define kEditSlot       3001
#define kCallSlot       3002

#define kAddConnection  4000
#define kEditConnection 4001
#define kDelConnection  4002

class TQQObjectGControl : public TQQGControl, public TQQObjectControl   {

 private:

  TString            fVisibleBaseClass;

  // Graphical ListTreeItems of Signals/Slot etc.

  TGListTreeItem    *fSignalsListTreeItem;  //!
  TGListTreeItem    *fSlotsListTreeItem;    //!
  TGListTreeItem    *fDelegationsListTreeItem; //!

  TGPopupMenu       *fSignalsListPopup;     //!
  TGPopupMenu       *fSlotsListPopup;       //!
  TGPopupMenu       *fDelegationsListPopup; //!

 public: 

  
  TQQObjectGControl(const TGWindow *parent=NULL,
		    UInt_t w=150, 
		    UInt_t h=150, 
		    UInt_t options = 0);
  
  // Build the signal/slot/delegation listtree into
  // the ListTree list starting at item "parent"

  void     ListTree(TGControlList  *list   = NULL,
		    TGListTreeItem *parent = NULL); 

  // Build the connections-listtree of the signal "signal"
  // into the listree starting at item

  virtual Bool_t UpdateConnections(TGControlList*,
				   TGListTreeItem *,
				   char *signal);

  virtual void UpdateDelegation(TGControlList*,
				TGListTreeItem *,
				char *delegation);


  virtual void SetObject(TQQObject *o);

  virtual void AddVisibleBaseClass(char *name) { fVisibleBaseClass = name;};
  
 
  // Slots
  

  virtual void Update();
  virtual void HandlePopup(Int_t);
  
  virtual void     TempSignal(){ Emit("TempSignal()");}; //*SIGNAL*
  virtual void     Paint() {     Emit("Paint()");};       //*SIGNAL*
  virtual void     NeedPaint() { Emit("NeedPaint()");};   //*SIGNAL*

  virtual ~TQQObjectGControl();
 

  ClassDef(TQQObjectGControl,1)

};

///////////////////////////////////////////////////////////////
//
// This makes the Dialog for a signal "signal" on the
// the Object "op"
//
///////////////////////////////////////////////////////////////

class SignalDialog : public TGTabDialog {
  // the operator control

  TQQObjectGControl   *fObjectControl;

  TGListTreeItem      *fItem;

  // the Delegation to process

  char                *fSignal;               

  // 1. Tab
  TGTextEntry         *fTxt1;
 public:
  SignalDialog(TQQObjectGControl*,
	       char*       ,
	       const TGWindow *p, 
	       const TGWindow *main, UInt_t w=500, UInt_t h=300,
	       UInt_t options = kVerticalFrame);
  ~SignalDialog();
  
  // Signals

  virtual void Apply();
 
};

///////////////////////////////////////////////////////////////
//
// This makes the Dialog for a slot "slot" on the
// the Object "op"
//
///////////////////////////////////////////////////////////////

class SlotDialog : public TGTabDialog {
  // the operator control

  TQQObjectGControl   *fObjectControl;
  TGListTreeItem      *fItem;

  // the Delegation to process

  char                *fSlot;               

  // 1. Tab
  TGTextEntry         *fTxt1;
 public:
  SlotDialog(TQQObjectGControl*,
	     char* ,
	     const TGWindow *p, 
	     const TGWindow *main, UInt_t w=500, UInt_t h=300,
	     UInt_t options = kVerticalFrame);
  ~SlotDialog();
  
  // Signals

  virtual void Apply();
 
};

///////////////////////////////////////////////////////////////
//
// This makes the Dialog for a connection of signal with "connection"
// "signal" 
//
///////////////////////////////////////////////////////////////

class ConnectionDialog : public TGTabDialog {
 
  // the operator control

  TQQObjectGControl   *fObjectControl;
  TGListTreeItem      *fItem;

  // the Signal to process

  char                *fSignal;               

  // the Connection to process
  
  char                *fConnection;

  // 1. Tab
  TGTextEntry         *fTxt1;
 public:
  ConnectionDialog(TQQObjectGControl*,
		   char*          signal,
		   char*          connection,
		   const TGWindow *p, 
		   const TGWindow *main, UInt_t w=500, UInt_t h=300,
		   UInt_t options = kVerticalFrame);
  ~ConnectionDialog();
  
  // Signals

  virtual void Apply();
 
};

///////////////////////////////////////////////////////////////
//
// This makes the Dialog for a delegation "delegation" on the
// the Object "op"
//
///////////////////////////////////////////////////////////////

class DelegationDialog : public TGTabDialog {  // the operator control

  TQQObjectGControl   *fObjectControl;
  TGListTreeItem      *fItem;

  // the Delegation to process

  char                *fDelegation;               

  // 1. Tab
  TGTextEntry         *fTxt1;
 public:
  DelegationDialog(TQQObjectGControl*,
		   char*           delegation,
		   const TGWindow *p, 
		   const TGWindow *main, UInt_t w=500, UInt_t h=300,
		   UInt_t options = kVerticalFrame);
  ~DelegationDialog();
  
  // Signals

  virtual void Apply();
 
};
#endif
