{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

///////////////////////////////////////////////////////////////////////////
//
// Name and Title of the Simulation
//
TString title="BPSK/NOFDM in an AWGN Channel";
TString name ="Bpsk_Awgn_GS";

cout << endl << title << endl << endl;

///////////////////////////////////////////////////////////////////////////
//
// Objects
//

TDSPFastBpsk         *qm = new TDSPFastBpsk("qm");
TDSPNofdm            *no = new TDSPNofdm("no");
TDSPAwgn             *aw = new TDSPAwgn("aw");
TDSPBlockSolve       *bs = new TDSPBlockSolve();

///////////////////////////////////////////////////////////////////////////
//
// Connections
//
qm->Connect(no,"Transmit(TDSPSignal*)");
no->Connect("Transmitted(TDSPSignal*)",aw,"Update(TDSPSignal*)");
aw->Connect(no,"Receive(TDSPSignal*)");
no->Connect("Received(TDSPSignal*)",bs,"Update(TDSPSignal*)");
bs->Connect(qm);


// Init the Gabor System
//
no->SetLength(80);
no->SetPads(8,8);
no->Init();

// Configure the NOFDM-Receiver to be a correlating receiver
//
no->SetOption("corr");

// Configure the iterative solver
//
TDSPLinearSolver *ls = new TDSPLinearSolver();
ls->SetStopPrecision(.00000000001);
ls->SetMaxIterations(30);
ls->SetMethod(kLinearSolveGaussSeidelBinary);
ls->SetA(no->GetCorrelation());
ls->SetSerialHD(9);
bs->SetSolver(ls);

cout << "bandwidth eff. = " << no->GetBandWidthEfficiency() << endl;

// Create a Window
//
TCanvas              *win  = new TCanvas(name,title,600,600);
TCanvas              *win1 = new TCanvas("iterations","mean iterations and precision",700,400);
win1->Divide(2,1);

win->SetGridx();
win->SetGridy();

// Create Theoretic Curve
//
#define num 100
#define min -3
#define max 14

Double_t ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  ser[i] = TMath::Log10(qm->SER(FromdB(snr[i])));
}

TGraph *fun1 = new TGraph(num,snr,ser);
fun1->SetName("Awgn_Ser");
fun1->SetLineStyle(2);
fun1->SetLineWidth(.1);
fun1->SetLineColor(2);


Double_t SNR_start   = 2;
Double_t SNR_end     = 20;
//Double_t SNR_end     = 28;
Double_t SNR_step    = 1;


Int_t    NumSymbols;
Int_t    MaxSignalLength = 100000;
cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;
Int_t    SignalLength;

TMultiGraph  tm;
tm.SetName("MultiGraph");
TGraphErrors Ser;
TGraph iter,prec;
iter.SetMarkerStyle(2);
iter.SetTitle("mean Number of Iterations");
prec.SetMarkerStyle(2);
prec.SetTitle("mean precision");

Ser.SetName("Ser");
Ser.SetTitle(title);
Ser.SetMarkerStyle(2);

tm.Add(&Ser,"cp");
tm.Add(fun1,"c");
tm.SetTitle(title);

Int_t  p=0;

for(Double_t SNR=SNR_start;SNR<SNR_end;SNR+=SNR_step) {

  // The Noise

  Double_t n=1./FromdB(SNR);
    
  // Calculate the number of needed Symbols
  // for a given Error of the SER

  NumSymbols = qm->EstimateNumSymbols(0.1, 100000,"serlog",10000);
 
  SignalLength = NumSymbols;  
  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;

  // Make them a multiple of the blocklen
  //
  NumSymbols   = no->GetBlockLen()*(   NumSymbols/no->GetBlockLen() );
  SignalLength = no->GetBlockLen()*( SignalLength/no->GetBlockLen() );

  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  
  // Set the Noise Power
  //
  aw->SetNoisePower(n);

  ls->ResetMeans();
  
  gBenchmark->Reset();
  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");
  
  cout << " NumSym="     << NumSymbols;
  cout << " Misid.= "    << qm->fMisIdentified; 
  cout << " Sin=" << aw->GetInputSignalPower();
  cout << " Sout=" << aw->GetOutputSignalPower();
  cout << " SNR = "      << dB(aw->GetSNR()) << " dB";
  cout << " SER = "      << qm->fLogSER;
  cout << " dSER = "     << qm->fdLogSER;
  cout << " time = "     << gBenchmark->GetCpuTime("qm") << endl;

  
  win->cd();
  win->Clear();


  if (qm->fMisIdentified) {

    iter.SetPoint(p      ,dB(aw->GetSNR()),ls->GetMeanIterations());
    prec.SetPoint(p      ,dB(aw->GetSNR()),ls->GetMeanPrecision());

    Ser.SetPoint(p       ,dB(aw->GetSNR()),qm->fLogSER);
    Ser.SetPointError(p++,0,qm->fdLogSER);

  
  }

  // Workaround for buggy TMultiGraph
  //
  if (tm.GetXaxis()) {
    a=tm.GetXaxis();
    b=tm.GetYaxis();
    a->SetLimits(a->GetXmin(),dB(aw->GetSNR())+.5);
    //b->SetLimits(Ser->GetMinimum(),Ser->GetMaximum());
    tm.GetXaxis()->SetTitle("SNR[dB]");
    tm.GetYaxis()->SetTitle("log SER");
  }

  tm.Draw("a");
  win->Update();

  //win1->Clear();
  win1->cd(1);
  iter.Draw("ac");
  win1->cd(2);
  //gPad->SetLogy();
  prec.Draw("ac");
  win1->Update();

  
}


TFile f(name+".root","RECREATE");
Ser.Write();
iter.Write();
prec.Write();
fun1->Write();
f.Close();
win->Print(name+".eps");

}
