{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << endl << "SER vs. SNR for QPSK/NOFDM in an AWGN Channel" << endl << endl;

TDSPPsk              *qm = new TDSPPsk("qm",4);
TDSPNofdm            *no = new TDSPNofdm("no");
TDSPAwgn             *aw = new TDSPAwgn("aw");
TDSPBlockSolve       *bs = new TDSPBlockSolve();


qm->Connect(no,"Transmit(TDSPSignal*)");
no->Connect("Transmitted(TDSPSignal*)",aw,"Update(TDSPSignal*)");
aw->Connect(no,"Receive(TDSPSignal*)");
no->Connect("Received(TDSPSignal*)",bs,"Update(TDSPSignal*)");
bs->Connect(qm);


no->SetLength(80);
no->SetPads(8,8);
no->Init();

TDSPLinearSolver     *ls = new TDSPLinearSolver();

TDSPMatrix *Gamma     = no->GetMatrix();         // That is Gamma
TDSPMatrix *GStar     = no->GetInverseMatrix();  // That is G'
TDSPMatrix *GammaStar = Gamma->Adjoint();       // That is Gamma'
TDSPMatrix *G         = GStar->Adjoint();       // That is G
// Calculate C=GammaStar*Channel*Gamma  Channel=1

TDSPMatrix  C(Gamma->GetCols(),Gamma->GetCols());
GammaStar->RightApply(Gamma,&C);

// Calculate C'
TDSPMatrix  *CStar = C.Adjoint();

// Calculate Q=GStar*G

TDSPMatrix  Q(Gamma->GetCols(),Gamma->GetCols());
GStar->RightApply(G,&Q);


TDSPMatrix  A(Gamma->GetCols(),Gamma->GetCols());


if (1) {
  gROOT->LoadMacro("sparsify.C+");
  sparsify(10,10,&A,&Q,&C,-1,1,-5,5);
  
} else {

  TDSPMatrix  *tmp = Q.RightApply(&C);
  tmp->LeftApply(CStar,&A);
  delete tmp;

}


// Configure the NOFDM-Receiver to be a Correlating Receiver
//
no->SetOption("corr");

// Configure the iterative Solver
//
TDSPVector *b   = new TDSPVector(100);
ls->SetStopPrecision(.001);
ls->SetMaxIterations(8);
ls->SetMethod(kLinearSolveGaussSeidelBinary);
ls->SetA(&A);
ls->SetB(b);
bs->SetSolver(ls);

//return;

// Create a Window
//
TCanvas              *win = new TCanvas("Qpsk","SER vs SNR for a Qpsk/NOFDM in an AWGN channel",600,600);

// Create Theoretic Curve
//
#define num 100
#define min -3
#define max 14

Double_t ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  ser[i] = TMath::Log10(qm->SER(FromdB(snr[i])));
}

TGraph *fun1 = new TGraph(num,snr,ser);
fun1->SetLineStyle(2);
fun1->SetLineWidth(.1);
fun1->SetLineColor(2);


Double_t start   = 0.00005;
//Double_t start   = 0.4;
Double_t end     = 0.5;
Double_t logstep = 1.25;

TH2F *Ser = new TH2F("Ser","SER vs. SNR",100,min,max,100,-1,0);
Ser->SetBit(TH1::kCanRebin);
Ser->GetXaxis()->SetTitle("SNR[dB]");
Ser->GetYaxis()->SetTitle("log SER");

Int_t    NumSymbols;
Int_t    MaxSignalLength = 100000;

cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;
Int_t    SignalLength;

for(Double_t n=end;n>=start;n/=logstep) {
    
  // Calculate the number of needed Symbols
  // for a given Error of the SER

  NumSymbols = qm->EstimateNumSymbols(0.05, 10000,"ser");
 
  SignalLength = NumSymbols;  
  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;

  // Make them a multiple of the blocklen
  //
  NumSymbols   = no->GetBlockLen()*(   NumSymbols/no->GetBlockLen() );
  SignalLength = no->GetBlockLen()*( SignalLength/no->GetBlockLen() );

  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  cout << " NumSym="     << NumSymbols << flush;
  // Set the Noise Power
  //
  aw->SetNoisePower(n);
  
  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");
  
  cout << " Misid.= "    << qm->fMisIdentified; 
  cout << " SNR = "      << dB(aw->GetSNR()) << " dB";
  cout << " SER = "      << qm->fInEfficiency;
  cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
  cout << " time = "     << gBenchmark->GetCpuTime("qm") << endl;

  // Fill and Draw Simulation Plot
  //
  Ser->Fill(dB(aw->GetSNR()),qm->fLogSER);
  Ser->Draw("box");
  
  // Include Theoretic Plot
  //
  fun1->Draw("C");
  
  gPad->Update();
  // win->Print("Qpsk_Awgn.eps");

}
}
