{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << endl << "SER vs. SNR for Qpsk/NOFDM in an Mult. Rayleigh Fading - AWGN Channel" << endl << endl;

TDSPPsk              *qm = new TDSPPsk("qm",4);
TDSPNofdm            *no = new TDSPNofdm("no");
TDSPAwgn             *aw = new TDSPAwgn("aw");
TDSPMultFading       *fa = new TDSPMultFading();

no->Init();


qm->Connect(no,"Transmit(TDSPSignal*)");
no->Connect("Transmitted(TDSPSignal*)",fa,"Update(TDSPSignal*)");
fa->Connect(aw);
aw->Connect(no,"Receive(TDSPSignal*)");
no->Connect("Received(TDSPSignal*)",   qm,"Update(TDSPSignal*)");



//
//////////////////////////////////////////////////////////////////////////
//

fa->SetMode(kRayleigh);
fa->SetMean(0);
fa->SetSigma(.1);
cout << "Gaussian with mean = " << fa->GetMeans() << " and sigma = " << fa->GetSigmas() << " (Rayleigh-Fading)" << endl;


//////////////////////////////////////////////////////////////////////////
//
// 128 Carrier
//
//no->SetBlockLen(6);
cout << "Using "<< no->GetNumCarriers() << " Carriers" << endl;

// Create a Window
//
TCanvas              *win = new TCanvas("Qpsk","SER vs SNR for a Qpsk/NOFDM in an Mult. Rayleigh-Fading-AWGN channel",650,650);


// Create Theoretic Curve
//
#define num 100
#define min -3
#define max 24

Double_t awgn_ser[100];
Double_t rayl_ser[100];
Double_t ofdm_rayl_ser[100];
Double_t snr[100];

Double_t pp = TMath::Sqrt(2*TMath::Pi());
Double_t  s = fa->GetSigmas().fRe;

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  awgn_ser[i] = TMath::Log10(qm->SER(FromdB(snr[i])));
  rayl_ser[i] = TMath::Log10(fa->SER(FromdB(snr[i]),qm));
  ofdm_rayl_ser[i] = TMath::Log10(qm->SER(1/(1/FromdB(snr[i])+(-pp*s+2*s*s+1)/0.02)));
}

TGraph *awgn_ser_model = new TGraph(num,snr,awgn_ser);
awgn_ser_model->SetLineStyle(2);
awgn_ser_model->SetLineWidth(.1);
awgn_ser_model->SetLineColor(2);


TGraph *rayl_ser_model = new TGraph(num,snr,rayl_ser);
rayl_ser_model->SetLineStyle(2);
rayl_ser_model->SetLineWidth(.1);
rayl_ser_model->SetLineColor(4);



TGraph *ofdm_rayl_ser_model = new TGraph(num,snr,ofdm_rayl_ser);
ofdm_rayl_ser_model->SetLineStyle(2);
ofdm_rayl_ser_model->SetLineWidth(.1);
ofdm_rayl_ser_model->SetLineColor(6);


Double_t start   = 0.0001;
Double_t end     = 0.3;
Double_t logstep = 1.35;

TH2F *Ser = new TH2F("Ser","SER vs. SNR",200,min,max,200,-1,0);
Ser->SetBit(TH2::kCanRebin|TH2::kNoStats);
Ser->GetXaxis()->SetTitle("SNR[dB]");
Ser->GetYaxis()->SetTitle("log SER");


Int_t    NumSymbols;
Int_t    MaxSignalLength = no->GetBlockLen()*( 100000/no->GetBlockLen() );

cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;
Int_t    SignalLength;


TH2D *histos[16];
Int_t hist_ind = 0;


for(Double_t rate=1;rate<=8;rate*=2) {

  qm->fInEfficiency=0;
  Ser->Reset();
  Ser->SetLineColor(hist_ind+1);

  fa->SetRate(rate/no->GetNumCarriers());
  cout << "Using a Fading of Rate so that NumCarrier x Rate = " << fa->GetRate()*no->GetNumCarriers() << endl;
  for(Double_t n=end;n>=start;n/=logstep) {
    
    // Calculate the number of needed Symbols
    // for a given Error of the SER
    
    NumSymbols = qm->EstimateNumSymbols(0.005, 100000,"logser");
    
    SignalLength = NumSymbols;  
    if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;
    
    // Make them a multiple of the blocklen
    //
    NumSymbols   = no->GetBlockLen()*(1+   NumSymbols/no->GetBlockLen() );
    SignalLength = no->GetBlockLen()*(1+ SignalLength/no->GetBlockLen() );
    
    // Set the Number of Symbols
    //
    qm->SetSignalLength(SignalLength);
    qm->SetAnalyseGrid( NumSymbols);
    
    // Set the Noise Power
    //
    aw->SetNoisePower(n);
    
    gBenchmark->Start("qm");
    qm->Generate();
    gBenchmark->Stop("qm");
    
    
    cout << " NumSym="     << NumSymbols;
    cout << " Misid.= "    << qm->fMisIdentified; 
    cout << " SNR = "      << dB(aw->GetSNR()) << " dB";
    cout << " SER = "      << qm->fInEfficiency;
    cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
    cout << " time = "     << gBenchmark->GetRealTime("qm") << endl;
    
    // Fill and Draw Simulation Plot
    //
    Ser->Fill(10*TMath::Log10(aw->GetSNR()),qm->fLogSER);
    Ser->Draw("box");
    
    // Include Theoretic Plot
    //
    awgn_ser_model->Draw("C");
    
    // Include Theoretic Rayleigh Plot
    //
    rayl_ser_model->Draw("C");

    // Include Theoretic Rayleigh Plot
    //
    //ofdm_rayl_ser_model->Draw("C");
    
    // win->Print("Ofdm_Qpsk_Awgn.eps");
   
    for(Int_t u = 0;u<hist_ind;u++) 
      histos[u]->Draw("box same");
   
    Ser->Draw("box same");
    

    gPad->Update();
    
    
  }

  
  cout << "Fading had a mean relative rate of " << fa->GetRelativeRate() << endl;
  

  // Copy the Histogram
  
  histos[hist_ind] = (TH2D*) Ser->Clone();
  //histos[hist_ind].SetLineColor(hist_ind+1);
  hist_ind++;
}

}
