{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << "Checking different AnalyseGrids for Qam4 in an AWGN-Channel and benchmarking \n\n";

TDSPQam              *qm = new TDSPQam("qm",4);
TDSPOfdm             *no = new TDSPOfdm("no");
TDSPAwgn             *aw = new TDSPAwgn("aw");


qm->Connect(no,"Transmit(TDSPSignal*)");
no->Connect("Transmitted(TDSPSignal*)",aw,"Update(TDSPSignal*)");
aw->Connect(no,"Receive(TDSPSignal*)");
no->Connect("Received(TDSPSignal*)",   qm,"Update(TDSPSignal*)");


// 128 Carrier
//
cout << "Using 128 Carrier" << endl;
no->SetBlockLen(128);

Double_t Noise = 0.04;
cout << "Setting Noise Level to " << Noise << endl;

Int_t    MaxSignalLength = 100000;

Int_t    SignalLength;

for(Int_t NumSymbols=1000;NumSymbols<10000000;NumSymbols*=2) {
    
 
  SignalLength = NumSymbols;

  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;

  // Make them a multiple of the blocklen
  //
  NumSymbols   = no->GetBlockLen()*(   NumSymbols/no->GetBlockLen() );
  SignalLength = no->GetBlockLen()*( SignalLength/no->GetBlockLen() );

  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  // Set the Noise Level
  //
  aw->SetNoiseLevel(Noise);
  
  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");
  
  cout << " NumSymbols=" << NumSymbols;
  cout << " SignalLen="  << SignalLength;
  cout << " Misid.= "    << qm->fMisIdentified; 
  cout << " SNR = "      << qm->fSNR;
  cout << " dB SER = "   << qm->fInEfficiency;
  cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
  cout << " time/sym = "     << gBenchmark->GetCpuTime("qm")/NumSymbols << endl;
  
}
}
