{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << endl << "SER vs. SNR for QAM4/OFDM - constant Multipath Channel + AWGN " << endl << endl;

//////////////////////////////////////////////////////////////////////////

TDSPQam              *qm  = new TDSPQam("qm",4);
TDSPOfdm             *no  = new TDSPOfdm("no");
TDSPAwgn             *aw  = new TDSPAwgn("aw");
TDSPConvolution *Channel  = new TDSPConvolution("Channel");// The Channel is simple convolution

//////////////////////////////////////////////////////////////////////////

qm->Connect(no,"Transmit(TDSPSignal*)");
no->Connect("Transmitted(TDSPSignal*)",Channel,"Update(TDSPSignal*)");
Channel->Connect(aw,"Update(TDSPSignal*)");
aw->Connect(no,"Receive(TDSPSignal*)");
no->Connect("Received(TDSPSignal*)",   qm,"Update(TDSPSignal*)");

//////////////////////////////////////////////////////////////////////////

// Create a Constant Multipath Channel Function, with 10 pads

Int_t pads = 129;
TDSPSignal *ChannelResponse = new TDSPSignal(pads);
  

// Create a Multipath Response Function
//
ChannelResponse->Zeros();
ChannelResponse(0)=1;
ChannelResponse(128)=.5;
ChannelResponse->Normalize();

Channel->Response(ChannelResponse);
//////////////////////////////////////////////////////////////////////////
//
// 128 Carrier
//
cout << "Using 128 Carrier\n";
no->SetBlockLen(128);
no->SetOverSampling(8);

// Create a Window
//
TCanvas              *win = new TCanvas("Qam4","SER vs SNR for a Qam4/OFDM in constant Multipath channel",600,600);

// Create Theoretic AWGN Curve
//
#define num 100
#define min -3
#define max 20

Double_t ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  ser[i] = qm->LogSER_dB(snr[i]);
}

TGraph *fun1 = new TGraph(num,snr,ser);
fun1->SetLineStyle(2);
fun1->SetLineWidth(.1);
fun1->SetLineColor(2);


Double_t start   = 0.0001;
Double_t end     = 0.5;
Double_t logstep = 1.05;

TH2F *Ser = new TH2F("Ser","SER vs. SNR",100,min,max,100,-1,0);
Ser->SetBit(TH1::kCanRebin);
Ser->GetXaxis()->SetTitle("SNR[dB]");
Ser->GetYaxis()->SetTitle("log SER");

Int_t    NumSymbols;
Int_t    MaxSignalLength = 100000;
cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;

Int_t    SignalLength;

for(Double_t n=end;n>=start;n/=logstep) {
    
  // Calculate the number of needed Symbols
  // for a given Error of the SER

  NumSymbols = qm->EstimateNumSymbols(0.05, 100000,"ser");
  
  SignalLength = NumSymbols;
  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;


  // Make them a multiple of the blocklen
  //
  NumSymbols   = no->GetBlockLen()*(   NumSymbols/no->GetBlockLen() );
  SignalLength = no->GetBlockLen()*( SignalLength/no->GetBlockLen() );

  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  // Set the Noise Power
  //
  aw->SetNoisePower(n);

  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");
  
  cout << " NumSym="   << NumSymbols;
  cout << " Misid="    << qm->fMisIdentified; 
  cout << " SNR="      << aw->GetSNR();
  cout << " SER="      << qm->fInEfficiency;
  cout << " dSER="     << 1/TMath::Sqrt(qm->fMisIdentified);
  cout << " time="     << gBenchmark->GetRealTime("qm") << endl;

  // Fill and Draw Simulation Plot
  //  
  Ser->Fill(10*TMath::Log10(aw->GetSNR()),qm->fLogSER);
  Ser->Draw("box");

  // Include Theoretic Plot
  //
  fun1->Draw("C");
  
  gPad->Update();
  // win->Print("Ofdm_Qam4_Channel.eps");

}
}
