{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << "\nSER vs. Clip-Level for QAM4 - AWGN Channel\n\n";

TDSPQam              *qm = new TDSPQam("qm",4);
TDSPOfdm             *no = new TDSPOfdm("no");
TDSPAwgn             *aw = new TDSPAwgn("aw");
TDSPClip             *cl = new TDSPClip("cl");


qm->Connect(no,"Transmit(TDSPSignal*)");
no->Connect("Transmitted(TDSPSignal*)",aw,"Update(TDSPSignal*)");
aw->Connect(cl);
cl->Connect(no,"Receive(TDSPSignal*)");
no->Connect("Received(TDSPSignal*)",   qm,"Update(TDSPSignal*)");


// 128 Carrier
//
cout << "Using 128 Carrier\n";
no->SetBlockLen(128);

// Create a Window
//
TCanvas              *win = new TCanvas("Qam4","SER vs SNR for a Qam4 in an AWGN channel",800,800);

// Create Theoretic Curve
//

#define num 100
#define min -3
#define max 14

Double_t ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  ser[i] = TMath::Log10(qm->SER(FromdB(snr[i])));
}

TGraph *fun1 = new TGraph(num,snr,ser);
fun1->SetLineStyle(2);
fun1->SetLineWidth(.1);
fun1->SetLineColor(2);


// Ranges for the Clip-Level
//

Double_t start   = 0.001;
Double_t end     = 1;
Double_t step    = .02;

// SER-SNR-Histogram
//
TH2F *Ser = new TH2F("Ser","SER vs. SNR",100,min,max,100,-1,0);
Ser->SetBit(TH1::kCanRebin);
Ser->GetXaxis()->SetTitle("SNR[dB]");
Ser->GetYaxis()->SetTitle("log SER");

/////////////////////////////////////////////////////
//
// Creating a Histogram for SER vs. ClipAmp


TProfile *SER_vs_Clip = new TProfile("SER_vs_Clip","SER vs. Clip-Level",(end-start)/step,start,end);
TProfile *SNR_vs_Clip = new TProfile("SNR_vs_Clip","SNR vs. Clip-Level",(end-start)/step,start,end);

SER_vs_Clip->GetXaxis()->SetTitle("clip level");
SER_vs_Clip->GetYaxis()->SetTitle("SER");
SNR_vs_Clip->GetXaxis()->SetTitle("clip level");
SNR_vs_Clip->GetYaxis()->SetTitle("SNR [db]");


/////////////////////////////////////////////////////


Int_t    NumSymbols;
Int_t    MaxSignalLength = 100000;

cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;
Int_t    SignalLength;

// Constant Noise

aw->SetNoisePower(.0001);

for(Double_t clip=end ; clip>=start ; clip-=step) {
  
  // Calculate the number of needed Symbols
  // for a given Error of the SER
  
  NumSymbols = qm->EstimateNumSymbols(0.05, 10000,"logser");

  SignalLength = NumSymbols;
  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;

  // Make them a multiple of the blocklen
  //
  NumSymbols   = no->GetBlockLen()*(   NumSymbols/no->GetBlockLen() );
  SignalLength = no->GetBlockLen()*( SignalLength/no->GetBlockLen() );

  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  // Set the Clipping Level
  //
  cl->SetClip(clip);
  
  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");
  
  cout << " Clip = "     << clip;
  cout << " NumSym="     << NumSymbols;
  cout << " SigLen="     << SignalLength;
  cout << " Misid.= "    << qm->fMisIdentified; 
  cout << " SNR = "      <<  dB(aw->GetSNR()) << " dB";
  cout << " SER = "      << qm->fInEfficiency;
  if (qm->fMisIdentified>0.0)
    cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
  cout << " time = "     << gBenchmark->GetRealTime("qm") << endl;

  win->cd(1);
  
  // Fill and Draw Simulation Plot
  //
  Ser->Fill(dB(aw->GetSNR()),qm->fLogSER);
  // Ser->Draw("box");
  
  // Include Theoretic Plot
  //
  // fun1->Draw("C");
  
  // win->cd(2);
  SER_vs_Clip->Fill(clip,qm->fLogSER);
  SER_vs_Clip->Draw("HISTL");
  
  // win->cd(3);
  // SNR_vs_Clip->Fill(clip,dB(aw->GetSNR()));
  // SNR_vs_Clip->Draw("HISTL");
  
  win->Update();
  // win->Print("Ofdm_Qam4_Clip_Awgn.eps");

}
}
