{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;
#include <iostream.h>

cout << endl << "SER vs. SNR for QAM4/OFDM - AWGN + Jakes Channel" << endl << endl;

//////////////////////////////////////////////////////////////////////////

TDSPQam              *qm  = new TDSPQam("qm",4);
TDSPOfdm             *no  = new TDSPOfdm("no");
TDSPAwgn             *aw  = new TDSPAwgn("aw");
TDSPConvolution *Channel  = new TDSPConvolution("Channel");// The Channel is simple convolution

//////////////////////////////////////////////////////////////////////////

qm->Connect(no,"Transmit(TDSPSignal*)");
no->Connect("Transmitted(TDSPSignal*)",Channel,"Update(TDSPSignal*)");
Channel->Connect(aw,"Update(TDSPSignal*)");
aw->Connect(no,"Receive(TDSPSignal*)");
no->Connect("Received(TDSPSignal*)",   qm,"Update(TDSPSignal*)");

//////////////////////////////////////////////////////////////////////////
//
// Create a Stochastic Channel Function, with 10 pads
//

Int_t pads = 10;
TDSPStochasticSignal *ChannelResponse = new TDSPStochasticSignal(pads);
  
// Create the Delta values 

ChannelResponse->Zeros();
ChannelResponse->SetAsModel();

// Creating the Correlator
//

TDSPJakes *Jakes = new TDSPJakes();

cout << "Configuring Jakes Model for the Stochastic Channel" << endl;
Jakes->SetCarrierFrequency(5.2e9);
cout << "  Carrier Frequ. = 5.2 GHz" << endl;
Jakes->SetSpeedKMH(20);
cout << "  Speed          = 20 km/h" << endl;
cout << "  Calculating ARMA-Model ";cout->flush();
Jakes->SetMethod(kCorrFilterProny);
Jakes->Init(.1,100);
cout << "done" << endl;
cout << "  Filternorm is  = " << Jakes->GetNorm() << endl;
ChannelResponse->SetFifoFilter(Jakes);
ChannelResponse->SetMode(kFilterGauss);
ChannelResponse->SetSigmas(.0001);
Channel->Response(ChannelResponse);
//////////////////////////////////////////////////////////////////////////
//
// 128 Carrier
//
cout << "Using 128 Carrier\n";
no->SetBlockLen(128);
// Create a Window
//
TCanvas              *win = new TCanvas("Qam4","SER vs SNR for a Qam4 in a Jakes channel",600,600);

// Create Theoretic Curve
//
#define num 100
#define min -3
#define max 14

Double_t ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  ser[i] = qm->LogSER_dB(snr[i]);
}

TGraph *fun1 = new TGraph(num,snr,ser);
fun1->SetLineStyle(2);
fun1->SetLineWidth(.1);
fun1->SetLineColor(2);


TH2D *cyclic[16];
Int_t hist_ind = 0;


TH2F *Ser = new TH2F("Ser","SER vs. SNR",100,min,max,100,-1,0);
Ser->SetBit(TH1::kCanRebin);
Ser->GetXaxis()->SetTitle("SNR[dB]");
Ser->GetYaxis()->SetTitle("log SER");

for(Int_t prefix=1;prefix<=129;prefix*=4) {

  qm->fInEfficiency=0;

  Ser->Reset();

  Ser->SetLineColor(hist_ind+1);
  Int_t cprefix = prefix==1 ? 0 : prefix;

  cout << endl << "Setting Cyclic Prefix to " << cprefix << endl;
  
  no->SetCyclicPrefix(cprefix);
  
  
  Double_t start   = 0.0001;
  Double_t end     = 0.5;
  Double_t logstep = 1.2;
  
  Int_t    NumSymbols; 
  Int_t    MaxSignalLength = 100000;
  
  Int_t    SignalLength;
  
  for(Double_t n=end;n>=start;n/=logstep) {
    
    // Calculate the number of needed Symbols
    // for a given Error of the SER
    
    
    NumSymbols = qm->EstimateNumSymbols(0.05, 100000,"logser");
    
    SignalLength = NumSymbols;
    if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;
    
    
    // Make them a multiple of the blocklen
    //
    NumSymbols   = no->GetBlockLen()*(   NumSymbols/no->GetBlockLen() );
    SignalLength = no->GetBlockLen()*( SignalLength/no->GetBlockLen() );
    
    // Set the Number of Symbols
    //
    qm->SetSignalLength(SignalLength);
    qm->SetAnalyseGrid( NumSymbols);
    
    // Set the Noise Power
    //
    aw->SetNoisePower(n);
    
    gBenchmark->Start("qm");
    qm->Generate();
    exit;
    gBenchmark->Stop("qm");
    
    cout << " NumSym="     << NumSymbols;
    cout << " Misid.= "    << qm->fMisIdentified; 
    cout << " SNR = "      << dB(aw->GetSNR()) << " dB";
    cout << " SER = "      << qm->fInEfficiency;
    cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
    cout << " time = "     << gBenchmark->GetRealTime("qm") << endl;
    
    // Fill and Draw Simulation Plot
    //
    Ser->Fill(10*TMath::Log10(aw->GetSNR()),qm->fLogSER);
    Ser->Draw("box");
    
    // Include Theoretic Plot
    //
    fun1->Draw("C");

    // Include Plots before ...

    for(Int_t u = 0;u<hist_ind;u++) 
      cyclic[u]->Draw("box same");
    
    Ser->Draw("box same");

    gPad->Update();
    // win->Print("Ofdm_Qam4_Jakes.eps");
    
  }

  // Copy the Histogram
  
  cyclic[hist_ind] = (TH2D*) Ser->Clone();
  //cyclic[hist_ind].SetLineColor(hist_ind+1);
  hist_ind++;
  
}

}

