{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << endl << "SER vs. SNR for QPSK/OFDM in an AWGN Channel" << endl << endl;

TDSPPsk              *qm = new TDSPPsk("qm",4);
TDSPOfdm             *no = new TDSPOfdm("no");
TDSPAwgn             *aw = new TDSPAwgn("aw");


qm->Connect(    "Update(TDSPSignal*)",     no,"Transmit(TDSPSignal*)");
no->Connect(    "Transmitted(TDSPSignal*)",aw,"Update(TDSPSignal*)");
aw->Connect(    "Update(TDSPSignal*)",     no,"Receive(TDSPSignal*)");
no->Connect(    "Received(TDSPSignal*)",   qm,"Update(TDSPSignal*)");


// 128 Carrier
//
cout << "Using 128 Carrier" << endl;
no->SetBlockLen(128);

// Create a Window
//
TCanvas              *win = new TCanvas("Qpsk","SER vs SNR for a QPSK/OFDM in an AWGN channel",600,600);

// Create Theoretic Curve
//
#define num 100
#define min 0
#define max 14

Double_t ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  ser[i] = qm->LogSER_dB(snr[i]);
}

TGraph *fun1 = new TGraph(num,snr,ser);
fun1->SetLineStyle(2);
fun1->SetLineWidth(.1);
fun1->SetLineColor(2);

Ser = new TGraphErrors();

Ser->SetName("Ser");
Ser->SetTitle("Symbol error rate");
Ser->SetMarkerStyle(2);

tm = new TMultiGraph();
tm->SetName("MultiGraph");
tm->SetTitle("OFDM/AWGN Symbol Error Rate");

tm->Add(Ser,"cp");
tm->Add(fun1,"c");

Double_t start   = 0.01;
Double_t end     = 0.5;
Double_t logstep = 1.05;

Double_t dError  = .05; // 5%
Bool_t   First_Time = kTRUE;
Int_t    NumSymbols = 10000; // Start

Int_t    MaxSignalLength = 100000;
cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;
Int_t    SignalLength;

Int_t  p=0;
for(Double_t n=end;n>=start;n/=logstep) {
    
  // Calculate the number of needed Symbols
  // for a given Error of the SER
  
  if (First_Time) First_Time = kFALSE;
  else     
    NumSymbols = 1/(qm->fInEfficiency*dError*dError);

  SignalLength = NumSymbols;
  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;

  // Make them a multiple of the blocklen
  //
  NumSymbols = no->GetBlockLen()*( NumSymbols/no->GetBlockLen() );
  SignalLength = no->GetBlockLen()*( SignalLength/no->GetBlockLen() );
    
  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  // Set the Noise Level
  //
  aw->SetNoiseLevel(n);
  
  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");


  if ((qm->fMisIdentified)&&(qm->fLogSER)) {
    Ser->SetPoint(p       ,dB(aw->GetSNR()),qm->fLogSER);
    Ser->SetPointError(p++,0,qm->fdLogSER);
  } 

  // Draw Simulation Plot
  //
  tm->Draw("a");
  if (tm->GetXaxis()) {
    tm->GetXaxis()->SetTitle("SNR[dB]");
    tm->GetYaxis()->SetTitle("log SER");
  }

  win->Update();

  cout << " NoisePower = "    <<  qm->fNoisePower;
  cout << " NumSym=" << NumSymbols;
  cout << " Misid.= "    << qm->fMisIdentified; 
  cout << " SNR = "      << dB(aw->GetSNR()) << " dB";
  cout << " SER = "      << qm->fInEfficiency;
  cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
  cout << " time = "     << gBenchmark->GetRealTime("qm") << endl;
  
  // win->Print("Ofdm_Qpsk_Awgn.eps");

}
}
