{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << endl << "SER vs. SNR for Bpsk - Mult. Rayleigh Fading - AWGN Channel" << endl << endl;

TDSPPsk              *qm = new TDSPPsk("qm",2);
TDSPAwgn             *aw = new TDSPAwgn("aw");
TDSPMultFading       *fa = new TDSPMultFading();

qm->Connect(fa);
fa->Connect(aw);
aw->Connect(qm);
//
//////////////////////////////////////////////////////////////////////////
//

fa->SetMode(kRayleigh);

// Create a Window
//
TCanvas              *win = new TCanvas("Bpsk","SER vs SNR for a Bpsk in an Mult. Rayleigh-Fading - AWGN channel",800,500);

// Create Theoretic Curve
//
#define num 100
#define min 0
#define max 24

Double_t awgn_ser[100];
Double_t rayl_ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  awgn_ser[i] = TMath::Log10(qm->SER(FromdB(snr[i])));
  rayl_ser[i] = TMath::Log10(fa->SER(FromdB(snr[i]),qm));
}

TGraph *awgn_ser_model = new TGraph(num,snr,awgn_ser);
awgn_ser_model->SetLineStyle(2);
awgn_ser_model->SetLineWidth(.1);
awgn_ser_model->SetLineColor(2);


TGraph *rayl_ser_model = new TGraph(num,snr,rayl_ser);
rayl_ser_model->SetLineStyle(2);
rayl_ser_model->SetLineWidth(.1);
rayl_ser_model->SetLineColor(4);

Double_t start   = 0.0001;
Double_t end     = 0.8;
Double_t logstep = 1.1;

TH2F *Ser = new TH2F("Ser","SER vs. SNR",100,min,max,100,-1,0);
Ser->SetBit(TH1::kCanRebin);

Ser->GetXaxis()->SetTitle("SNR[dB]");
Ser->GetYaxis()->SetTitle("log SER");


Int_t    MaxSignalLength = 100000;

cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;
Int_t    SignalLength;

for(Double_t n=end;n>=start;n/=logstep) {
    
  // Calculate the number of needed Symbols
  // for a given Error of the SER. If no information is avaiable 
  // use 10000
  
  NumSymbols = qm->EstimateNumSymbols(0.05, 100000,"logser");
  
  SignalLength = NumSymbols;
  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;
    
  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  // Set the Noise Level
  //
  aw->SetNoisePower(n);
  
  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");


  // Draw Simulation Plot
  //
  
  cout << " NumSym="        << NumSymbols;
  cout << " Misid.= "    << qm->fMisIdentified; 
  cout << " SNR = "      << dB(aw->GetSNR()) << " dB";
  cout << " SER = "      << qm->fInEfficiency;
  cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
  cout << " time = "     << gBenchmark->GetRealTime("qm") << endl;

  // Fill the SER vs SNR Histogram
  //
  Ser->Fill(dB(aw->GetSNR()),qm->fLogSER);
  Ser->Draw("box");

  // Include Theoretic AWGN Plot
  //
  awgn_ser_model->Draw("C");

  // Include Theoretic Rayleigh Plot
  //
  rayl_ser_model->Draw("C");

  
  
  win->Update();
  
  // win->Print("Bpsk_Rayleigh.eps");

}
}
