{

gROOT->Reset();
gErrorIgnoreLevel = kWarning;

#include <iostream.h>

cout << "\nSER vs. SNR for Pam2 - AWGN Channel\n\n";

TDSPPam              *qm = new TDSPPam("qm",2);
TDSPAwgn             *aw = new TDSPAwgn("aw");


qm->Connect(aw);
aw->Connect(qm);


// Create a Window
//
TCanvas              *win = new TCanvas("Pam2","SER vs SNR for a Pam2 in an AWGN channel",600,600);

// Create Theoretic Curve
//
#define num 100
#define min 0
#define max 14

Double_t ser[100];
Double_t snr[100];

for(Int_t i=0;i<num;i++) {
  snr[i] = (max-min)/((Double_t)num)*i+min;
  ser[i] = TMath::Log10(qm->SER(FromdB(snr[i])));
}

TGraph *fun1 = new TGraph(num,snr,ser);
fun1->SetLineStyle(2);
fun1->SetLineWidth(.1);
fun1->SetLineColor(2);


Double_t start   = 0.0001;
Double_t end     = 0.5;
Double_t logstep = 1.05;


TH2F *Ser = new TH2F("Ser","SER vs. SNR",100,min,max,100,-1,0);
Ser->SetBit(TH1::kCanRebin);

Ser->GetXaxis()->SetTitle("SNR[dB]");
Ser->GetYaxis()->SetTitle("log SER");

Int_t    NumSymbols; 
Int_t    MaxSignalLength = 100000;

cout << "Setting the maximal signal length to : " << MaxSignalLength << endl;
Int_t    SignalLength;

for(Double_t n=end;n>=start;n/=logstep) {
    
  // Calculate the number of needed Symbols
  // for a given Error of the SER
      
  NumSymbols = qm->EstimateNumSymbols(0.05, 10000,"ser");

  SignalLength = NumSymbols;
  if (SignalLength>MaxSignalLength) SignalLength=MaxSignalLength;
    
  // Set the Number of Symbols
  //
  qm->SetSignalLength(SignalLength);
  qm->SetAnalyseGrid( NumSymbols);
   
  // Set the Noise Power
  //
  aw->SetNoisePower(n);
  
  gBenchmark->Start("qm");
  qm->Generate();
  gBenchmark->Stop("qm");


  
  cout << " NumSym="      << NumSymbols;
  cout << " Misid.= "    << qm->fMisIdentified; 
  cout << " SNR = "      << dB(aw->GetSNR()) << " dB";
  cout << " SER = "      << qm->fInEfficiency;
  cout << " dSER = "     << 1/TMath::Sqrt(qm->fMisIdentified);
  cout << " time = "     << gBenchmark->GetRealTime("qm") << endl;
  
  // Fill and Draw Simulation Plot
  //
  Ser->Fill(dB(aw->GetSNR()),qm->fLogSER);
  Ser->Draw("box");
  
  // Include Theoretic Plot
  //
  fun1->Draw("C");
  
  gPad->Update();
  
  // win->Print("Pam2_Awgn.eps");

}
}
