{

gROOT->Reset();
gErrorIgnoreLevel = kError;

// The Symbol Generator should be Qam16

TDSPQam         *Qam      = new TDSPQam("Qam",16);
Qam->SetSignalLength(100);
Qam->SetAnalyseGrid(100);

// The Channel is simple convolution

TDSPConvolution *Channel  = new TDSPConvolution("Channel");

// Connect them

Qam->Connect(    "Update(TDSPSignal*)", Channel, "Update(TDSPSignal*)");
Channel->Connect("Update(TDSPSignal*)", Qam    , "Update(TDSPSignal*)");

// Create a Stochastic Channel Function, with 10 pads

Int_t pads = 10;
TDSPStochasticSignal *ChannelResponse = new TDSPStochasticSignal(pads);

// Create the Delta values 

ChannelResponse->Delta();

cout << "Creating a Delta channel function with " << pads << " pads" << endl;
cout << "ChannelResponse = " << endl;
ChannelResponse->Print();

cout << "Using this as Model (as the mean values...)" << endl;
ChannelResponse->SetAsModel();
  
// Creating the Correlator


TDSPFilter     *jakes      = new TDSPFilter();

cout << "Creating a FIR/IIR-Filter (Jakes 10 km/h)" << endl;
cout << "FIR=" << endl;

// The fir-part of the filter

TDSPSignal *fir = new TDSPVector(2);
fir(0)=1;
fir(1)=-0.8405;

fir->Print();

// the iir-part of the filter

cout << "FIR=" << endl;

TDSPSignal *iir = new TDSPVector(4);
iir(0)=1;
iir(1)=-1.8178;
iir(2)=0.8689;
iir(3)=0.0205;

iir->Print();

jakes->FIR(fir);
jakes->IIR(iir);
  
TDSPBlockFifoFilter *correlator = new TDSPBlockFifoFilter();
correlator->SetFilter(jakes);

ChannelResponse->SetFifoFilter(correlator);

ChannelResponse->SetMode(kFilterGauss);

// Configure the stochastic Channel Response to be gaussian, BUT correlated
// with a special input sigma

Double_t sigma = .02;
cout << "Setting now this to a gaussian stochastic process with sigma = " << sigma << endl;

ChannelResponse->SetSigmas(TComplex(sigma,sigma));

// Connect the Stochastic ChannelResponse to the Channel

Channel->Response(ChannelResponse);

// Create some in/out views

TDSPSignalView  *Input    = new TDSPSignalView("Input", "Input Signal", "real ab");
TDSPSignalView  *Output   = new TDSPSignalView("Output","Output Signal","complex a*");

// Connect the views

Qam->Connect(    "Update(TDSPSignal*)", Input,   "Update(TDSPSignal*)");
Channel->Connect("Update(TDSPSignal*)", Output,  "Update(TDSPSignal*)");


for(Int_t i=0;i<100;i++) {    
  Qam->Generate();
}

}
